@@ -9,6 +9,9 @@
 #include <vector>
 #include <string>
 
+#include "CGNSOptions.h"
+#include "PartitionOptions.h"
+
 // Interface-independent context 
 
 class Context_T {
@@ -47,6 +50,7 @@ class Context_T {
   int gl_position[2]; // position of the graphic window on the screen
   int msg_position[2]; // position of the message window on the screen
   int msg_size[2]; // size of the message window on the screen
+  int msg_auto_scroll; // scroll automatically to last message
   int opt_position[2]; // position of the option window on the screen
   int vis_position[2]; // position of the visibility window on the screen
   int clip_position[2]; // position of the clipping planes window on the screen
@@ -60,7 +64,9 @@ class Context_T {
   int field_size[2]; // size of the field window on the screen
   int file_chooser_position[2]; // position of the file chooser window on the screen
   int system_menu_bar; // use the system menu bar on MacOS?
-  int batch; // 0=full gfx; -1=just parse; 1,2,3=batch 1D, 2D, 3D mesh 
+  int batch; // -3=server daemon; -2=check coherence; -1=write geo; 0=full gfx;
+             // 1=1D mesh; 2=2D mesh; 3=3D mesh; 4=adapt mesh; 5=refine mesh;
+  int batch_after_mesh; // 1=partition mesh
   int initial_context; // 0=automatic; 1=geom; 2=mesh; 3=solver; 4=post 
   int nopopup; // never popup dialogs in scripts (use default values instead)
   int non_modal_windows; // make all windows "non modal"
@@ -123,6 +129,7 @@ class Context_T {
   int expert_mode; // to disable some warnings for beginners
   int printing; // dynamic: equal to 1 while gmsh is printing
   int hide_unselected; // hide all unselected entities
+  int draw_all_models;
 
   // geometry options 
   struct{
@@ -154,21 +161,24 @@ class Context_T {
     int optimize, optimize_netgen, refine_steps;
     int quality_type, label_type;
     double quality_inf, quality_sup, radius_inf, radius_sup;
-    double scaling_factor, lc_factor, rand_factor, lc_integration_precision, lc_min, lc_max;
+    double scaling_factor, lc_factor, rand_factor, lc_integration_precision, lc_min, lc_max, tolerance_edge_length;
     int lc_from_points, lc_from_curvature, lc_extend_from_boundary;
-    int dual, draw_skin_only;
+    int dual, voronoi, draw_skin_only;
     int light, light_two_side, light_lines;
     int format, nb_smoothing, algo2d, algo3d, algo_recombine;
     int order, second_order_linear, second_order_incomplete;
-    int second_order_experimental;
+    int second_order_experimental, mesh_only_visible;
     int smooth_internal_edges, c1_continuity;
     int min_circ_points, min_curv_points;
     double normals, tangents, explode;
     int color_carousel;
-    int save_all, save_groups_of_nodes, stl_binary, msh_binary, bdf_field_format;
+    int save_all, save_groups_of_nodes, binary, bdf_field_format, save_parametric;
     int smooth_normals, reverse_all_normals;
     double angle_smooth_normals;
     double allow_swap_edge_angle;
+    int zone_definition;
+    PartitionOptions partition_options;
+    CGNSOptions cgns_options;
   } mesh;
 
   // post processing options 
@@ -195,7 +205,7 @@ class Context_T {
     int jpeg_quality, jpeg_smoothing;
     int gif_dither, gif_sort, gif_interlace, gif_transparent;
     int geo_labels;
-    int pos_elementary, pos_element, pos_gamma, pos_eta, pos_rho;
+    int pos_elementary, pos_element, pos_gamma, pos_eta, pos_rho,pos_disto;
     int text, tex_as_equation;
   } print;
 
