@@ -95,7 +95,7 @@ StringXString MeshOptions_String[] = {
 StringXString SolverOptions_String[] = {
   { F|O, "SocketName" , opt_solver_socket_name , 
 #if defined(WIN32) && !defined(__CYGWIN__)
-    "127.0.0.1:44122" , // use TCP/IP sockets by default on "pure" Windows
+    "127.0.0.1:44122" , // use TCP/IP sockets by default on Windows
 #else
     ".gmshsock" , // otherwise use Unix sockets by default
 #endif
@@ -543,6 +543,8 @@ StringXNumber GeneralOptions_Number[] = {
 
   { F|O, "DoubleBuffer" , opt_general_double_buffer , 1. ,
     "Use a double buffered graphic window (on Unix, should be set to 0 when working on a remote host without GLX)" },
+  { F|O, "DrawAllModels" , opt_general_draw_all_models, 0. , 
+    "Draw all loaded models (instead of drawing only the current one)" },
   { F|O, "DrawBoundingBoxes" , opt_general_draw_bounding_box, 0. ,
     "Draw bounding boxes" },
 
@@ -657,6 +659,8 @@ StringXNumber GeneralOptions_Number[] = {
     "Horizontal position (in pixels) of the upper left corner of the menu window" }, 
   { F|S, "MenuPositionY" , opt_general_menu_position1 , 50. ,
     "Vertical position (in pixels) of the upper left corner of the menu window" }, 
+  { F|S, "MessageAutoScroll" , opt_general_message_auto_scroll , 1. , 
+    "Automatically scroll message window" }, 
   { F|S, "MessagePositionX" , opt_general_message_position0 , 650. , 
     "Horizontal position (in pixels) of the upper left corner of the message window" }, 
   { F|S, "MessagePositionY" , opt_general_message_position1 , 490. , 
@@ -784,7 +788,7 @@ StringXNumber GeneralOptions_Number[] = {
 
   { F|O, "VectorType" , opt_general_vector_type , 4 ,
     "Default vector display type (for normals, etc.)" },
-  { F|O, "Verbosity" , opt_general_verbosity , 3. ,
+  { F|O, "Verbosity" , opt_general_verbosity , 4. ,
     "Level of information printed during processing (0=no information)" },
   { F|S, "VisibilityPositionX" , opt_general_visibility_position0 , 650. , 
     "Horizontal position (in pixels) of the upper left corner of the visibility window" }, 
@@ -865,7 +869,7 @@ StringXNumber GeometryOptions_Number[] = {
   { F|O, "SurfaceNumbers" , opt_geometry_surfaces_num , 0. , 
     "Display surface numbers?" },
   { F|O, "SurfaceType" , opt_geometry_surface_type , 2. , 
-    "Surface display type (0=cross, 1=wireframe, 2=solid" },
+    "Surface display type (0=cross, 1=wireframe, 2=solid)" },
 
   { F|O, "Tangents" , opt_geometry_tangents , 0. ,
     "Display size of tangent vectors (in pixels)" },
@@ -882,7 +886,7 @@ StringXNumber GeometryOptions_Number[] = {
 
 StringXNumber MeshOptions_Number[] = {
   { F|O, "Algorithm" , opt_mesh_algo2d , ALGO_2D_MESHADAPT_DELAUNAY ,
-    "2D mesh algorithm (1=MeshAdapt+Delaunay, 4=MeshAdapt, 5=Delaunay)" }, 
+    "2D mesh algorithm (1=MeshAdapt+Delaunay, 4=MeshAdapt, 5=Delaunay, 6=Frontal)" }, 
   { F|O, "Algorithm3D" , opt_mesh_algo3d , 
 #if defined(HAVE_TETGEN)
     ALGO_3D_TETGEN_DELAUNAY ,
@@ -897,16 +901,50 @@ StringXNumber MeshOptions_Number[] = {
 
   { F|O, "BdfFieldFormat" , opt_mesh_bdf_field_format , 1. , 
     "Field format for Nastran BDF files (0=free, 1=small, 2=large)" },
+  { F|O, "Binary" , opt_mesh_binary , 0. , 
+    "Write mesh files in binary format (if possible)" },
 
   { F|O, "C1Continuity" , opt_mesh_c1 , 0. ,
     "Impose C1 continuity to high order meshes (only valid in 2D plane and ElemenOrder = 2)" },
+  { F|O, "ChacoArchitecture" , opt_mesh_partition_chaco_architecture, 1. ,
+    "(Adv. Chaco): Parallel architecture topology (0=hypercube, 1-3=mesh dimensions)" },
+  { F|O, "ChacoEigensolver" , opt_mesh_partition_chaco_eigensolver, 1. ,
+    "(Adv. Chaco): Type of eigensolver for a spectral algorithm (0=Lanczos, 1=Multilevel RQI/Symmlq)" },
+  { F|O, "ChacoEigTol" , opt_mesh_partition_chaco_eigtol, 1.E-3 ,
+    "(Adv. Chaco): Tolerance of the eigensolver for spectral or multilevel-KL algorithms" },
+  { F|O, "ChacoGlobalMethod" , opt_mesh_partition_chaco_global_method, 1. ,
+    "Chaco partitioning algorithm (1=Multilevel-KL, 2=Spectral, 4=Linear, 5=Random, 6=Scattered)" },
+  { F|O, "ChacoHypercubeDim" , opt_mesh_partition_chaco_ndims_tot, 2. ,
+    "(Adv. Chaco): Dimensional partitioning for a hypercube topology" },
+  { F|O, "ChacoLocalMethod" , opt_mesh_partition_chaco_local_method, 1. ,
+    "(Adv. Chaco): Local partitioning algorithm" },
+  { F|O, "ChacoMeshDim1" , opt_mesh_partition_chaco_mesh_dims1, 4. ,
+    "(Adv. Chaco): Number of partitions in the first dimension of a mesh topology" },
+  { F|O, "ChacoMeshDim2" , opt_mesh_partition_chaco_mesh_dims2, 1. ,
+    "(Adv. Chaco): Number of partitions in the second dimension of a mesh topology" },
+  { F|O, "ChacoMeshDim3" , opt_mesh_partition_chaco_mesh_dims3, 1. ,
+    "(Adv. Chaco): Number of partitions in the third dimension of a mesh topology" },
+  { F|O, "ChacoPartitionSection" , opt_mesh_partition_chaco_nsection, 1. ,
+    "(Adv. Chaco): Partition by (1=bisection, 2=quadrisection, 3=octasection" },
+  { F|O, "ChacoSeed" , opt_mesh_partition_chaco_seed, 7654321. ,
+    "(Adv. Chaco): Seed for random number generator" },
+  { F|O, "ChacoVMax" , opt_mesh_partition_chaco_vmax, 250. ,
+    "(Adv. Chaco): Maximum vertices in a coarse graph (for multilevel-KL algorithm and Multilevel RQI/Symmlq eigensolver)" },
+  { F|O, "ChacoParamINTERNAL_VERTICES" , opt_mesh_partition_chaco_internal_vertices, 0. ,
+    "(Adv. Chaco): Parameter INTERNAL_VERTICES" },
+  { F|O, "ChacoParamREFINE_MAP" , opt_mesh_partition_chaco_refine_map, 1. ,
+    "(Adv. Chaco): Parameter REFINE_MAP" },
+  { F|O, "ChacoParamREFINE_PARTITION" , opt_mesh_partition_chaco_refine_partition, 0. ,
+    "(Adv. Chaco): Parameter REFINE_PARTITION" },
+  { F|O, "ChacoParamTERMINAL_PROPOGATION" , opt_mesh_partition_chaco_terminal_propogation, 0. ,
+    "(Adv. Chaco): Parameter TERMINAL_PROPOGATION" },
   { F|O, "CharacteristicLengthExtendFromBoundary" , opt_mesh_lc_extend_from_boundary, 1. ,
     "Extend characteristic lengths from the boundaries inside the surface/volume" },
   { F|O, "CharacteristicLengthFactor" , opt_mesh_lc_factor , 1.0 ,
     "Factor applied to all characteristic lengths" },
   { F|O, "CharacteristicLengthMin" , opt_mesh_lc_min, 0.0 ,
     "Minimum characteristic length" },
-  { F|O, "CharacteristicLengthMax" , opt_mesh_lc_max, 1.e22 ,
+  { F|O, "CharacteristicLengthMax" , opt_mesh_lc_max, 1.e22,
     "Maximum characteristic length" },
   { F|O, "CharacteristicLengthFromCurvature" , opt_mesh_lc_from_curvature , 0. ,
     "Compute characteristic lengths from curvature" },
@@ -950,12 +988,18 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "LineWidth" , opt_mesh_line_width , 1.0 , 
     "Display width of mesh lines (in pixels)" },
 
+  { F|O, "MeshOnlyVisible" , opt_mesh_mesh_only_visible, 0. ,
+    "Mesh only visible entities (experimental: use with caution!)" },
+  { F|O, "MetisAlgorithm" , opt_mesh_partition_metis_algorithm, 1. ,
+    "METIS partitioning algorithm (1=Recursive, 2=K-way)" },
+  { F|O, "MetisEdgeMatching" , opt_mesh_partition_metis_edge_matching, 3. ,
+    "(Adv. METIS): Determines the matching type (1=Random, 2=Heavy-Edge, 3=Sorted Heavy-Edge)" },
+  { F|O, "MetisRefinementAlgorithm" , opt_mesh_partition_metis_refine_algorithm, 3. ,
+    "(Adv. METIS): Algorithm for k-way refinement (1=Random, 2=Greedy, 3=Random with minimized connectivity)" },
   { F|O, "MinimumCirclePoints" , opt_mesh_min_circ_points, 7. ,
     "Minimum number of points used to mesh a circle" },
   { F|O, "MinimumCurvePoints" , opt_mesh_min_curv_points, 3. ,
     "Minimum number of points used to mesh a (non-straight) curve" },
-  { F|O, "MshBinary" , opt_mesh_msh_binary , 0. , 
-    "Write MSH files in binary format?" },
   { F|O, "MshFileVersion" , opt_mesh_msh_file_version , 2.0 , 
     "Version of the MSH file format to use" },
 
@@ -963,6 +1007,8 @@ StringXNumber MeshOptions_Number[] = {
     "Number of hexahedra in the current mesh (read-only)" },
   { F, "NbNodes" , opt_mesh_nb_nodes , 0. , 
     "Number of nodes in the current mesh (read-only)" },
+  { F|O, "NbPartitions" , opt_mesh_partition_num, 4. ,
+    "Number of partitions" },
   { F, "NbPrisms" , opt_mesh_nb_prisms , 0. , 
     "Number of prisms in the current mesh (read-only)" },
   { F, "NbPyramids" , opt_mesh_nb_pyramids , 0. , 
@@ -981,6 +1027,8 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "OptimizeNetgen" , opt_mesh_optimize_netgen , 0. , 
     "Optimize the mesh using Netgen to improve the quality of tetrahedral elements" },
 
+  { F|O, "Partitioner" , opt_mesh_partition_partitioner, 2. ,
+    "Partitioner software (1=Chacho, 2=METIS)" },
   { F|O, "Points" , opt_mesh_points , 0. , 
     "Display mesh vertices (nodes)?" },
   { F|O, "PointNumbers" , opt_mesh_points_num , 0. , 
@@ -1018,6 +1066,8 @@ StringXNumber MeshOptions_Number[] = {
 
   { F,   "SaveAll" , opt_mesh_save_all , 0. , 
     "Ignore Physical definitions and save all elements" },
+  { F,   "SaveParametric" , opt_mesh_save_parametric , 0. , 
+    "Save parametric coordinates of nodes" },
   { F,   "SaveGroupsOfNodes" , opt_mesh_save_groups_of_nodes , 0. , 
     "Save groups of nodes for each physical line and surface (UNV mesh format only)" },
   { F|O, "ScalingFactor" , opt_mesh_scaling_factor , 1.0 ,
@@ -1036,8 +1086,6 @@ StringXNumber MeshOptions_Number[] = {
     "Number of smoothing steps of internal edges for high order meshes" },
   { F|O, "SmoothNormals" , opt_mesh_smooth_normals , 0. , 
     "Smooth the mesh normals?" },
-  { F|O, "StlBinary" , opt_mesh_stl_binary , 0. , 
-    "Save STL files in binary format?" },
   { F|O, "SurfaceEdges" , opt_mesh_surfaces_edges , 1. , 
     "Display edges of surface mesh?" },
   { F|O, "SurfaceFaces" , opt_mesh_surfaces_faces , 0. , 
@@ -1049,6 +1097,8 @@ StringXNumber MeshOptions_Number[] = {
     "Display size of tangent vectors (in pixels)" }, 
   { F|O, "Tetrahedra" , opt_mesh_tetrahedra , 1. , 
     "Display mesh tetrahedra?" },
+  { F|O, "ToleranceEdgeLength" , opt_mesh_tolerance_edge_length, 0.0,
+    "Skip a model edge in mesh generation if its length is less than user's defined tolerance" },
   { F|O, "Triangles" , opt_mesh_triangles , 1. , 
     "Display mesh triangles?" },
 
@@ -1058,6 +1108,11 @@ StringXNumber MeshOptions_Number[] = {
     "Display faces of volume mesh?" },
   { F|O, "VolumeNumbers" , opt_mesh_volumes_num , 0. , 
     "Display volume mesh element numbers?" },
+  { F|O, "Voronoi" , opt_mesh_voronoi , 0. ,
+    "Display the voronoi diagram" },
+
+  { F|O, "ZoneDefinition" , opt_mesh_zone_definition , 0. , 
+    "Method for defining a zone (0=single zone, 1=by partition, 2=by physical)" },
 
   { 0, NULL , NULL , 0. , NULL }
 } ;
@@ -1428,6 +1483,8 @@ StringXNumber PrintOptions_Number[] = {
     "Save Eta quality measure in mesh statistics exported as post-processing views" },
   { F|O, "PostRho" , opt_print_pos_rho , 0. ,
     "Save Rho quality measure in mesh statistics exported as post-processing views" },
+  { F|O, "PostDisto" , opt_print_pos_disto , 0. ,
+    "Save Disto quality measure in mesh statistics exported as post-processing views" },
 
   { F|O, "TexAsEquation" , opt_print_tex_as_equation , 0. ,
     "Print all TeX strings as equations" },
