@@ -7,18 +7,13 @@
 #define _GMSH_H_
 
 #include <string>
-
-class GmshMessage{
- public:
-  GmshMessage(){}
-  virtual ~GmshMessage(){}
-  virtual void operator()(std::string level, std::string message){}
-};
+#include "GmshMessage.h"
 
 int GmshInitialize(int argc=0, char **argv=0);
 int GmshSetMessageHandler(GmshMessage *callback);
 int GmshSetOption(std::string category, std::string name, std::string value, int index=0);
 int GmshSetOption(std::string category, std::string name, double value, int index=0);
+int GmshMergeFile(std::string fileName);
 int GmshFinalize();
 int GmshBatch();
 
