@@ -8,15 +8,8 @@
 
 #if defined(HAVE_FLTK)
 #  include <FL/Fl.H>
-#  if (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION == 1)
-#    if (FL_PATCH_VERSION >= 6)
-#      define HAVE_FLTK_1_1_6_OR_ABOVE
-#    endif
-#    if (FL_PATCH_VERSION >= 5)
-#      define HAVE_FLTK_1_1_5_OR_ABOVE
-#    endif
-#  else
-#    error "Gmsh requires FLTK 1.1.x"
+#  if (FL_MAJOR_VERSION != 1) || (FL_MINOR_VERSION != 1) || (FL_PATCH_VERSION < 7)
+#    error "Gmsh requires FLTK 1.1.7 or above"
 #  endif
 #  include <FL/gl.h>
 #  if defined(__APPLE__)
