@@ -234,6 +234,7 @@ double opt_general_message_position0(OPT_ARGS_NUM);
 double opt_general_message_position1(OPT_ARGS_NUM);
 double opt_general_message_size0(OPT_ARGS_NUM);
 double opt_general_message_size1(OPT_ARGS_NUM);
+double opt_general_message_auto_scroll(OPT_ARGS_NUM);
 double opt_general_option_position0(OPT_ARGS_NUM);
 double opt_general_option_position1(OPT_ARGS_NUM);
 double opt_general_plugin_position0(OPT_ARGS_NUM);
@@ -287,6 +288,7 @@ double opt_general_orthographic(OPT_ARGS_NUM);
 double opt_general_mouse_selection(OPT_ARGS_NUM);
 double opt_general_mouse_hover_meshes(OPT_ARGS_NUM);
 double opt_general_draw_bounding_box(OPT_ARGS_NUM);
+double opt_general_draw_all_models(OPT_ARGS_NUM);
 double opt_general_fast_redraw(OPT_ARGS_NUM);
 double opt_general_xmin(OPT_ARGS_NUM);
 double opt_general_xmax(OPT_ARGS_NUM);
@@ -429,6 +431,7 @@ double opt_mesh_explode(OPT_ARGS_NUM);
 double opt_mesh_scaling_factor(OPT_ARGS_NUM);
 double opt_mesh_lc_min(OPT_ARGS_NUM);
 double opt_mesh_lc_max(OPT_ARGS_NUM);
+double opt_mesh_tolerance_edge_length(OPT_ARGS_NUM);
 double opt_mesh_lc_factor(OPT_ARGS_NUM);
 double opt_mesh_lc_from_curvature(OPT_ARGS_NUM);
 double opt_mesh_lc_from_points(OPT_ARGS_NUM);
@@ -468,13 +471,13 @@ double opt_mesh_light_lines(OPT_ARGS_NUM);
 double opt_mesh_light_two_side(OPT_ARGS_NUM);
 double opt_mesh_format(OPT_ARGS_NUM);
 double opt_mesh_msh_file_version(OPT_ARGS_NUM);
-double opt_mesh_msh_binary(OPT_ARGS_NUM);
-double opt_mesh_stl_binary(OPT_ARGS_NUM);
+double opt_mesh_binary(OPT_ARGS_NUM);
 double opt_mesh_bdf_field_format(OPT_ARGS_NUM);
 double opt_mesh_nb_smoothing(OPT_ARGS_NUM);
 double opt_mesh_algo2d(OPT_ARGS_NUM);
 double opt_mesh_algo3d(OPT_ARGS_NUM);
 double opt_mesh_recombine_algo(OPT_ARGS_NUM);
+double opt_mesh_mesh_only_visible(OPT_ARGS_NUM);
 double opt_mesh_min_circ_points(OPT_ARGS_NUM);
 double opt_mesh_allow_swap_edge_angle(OPT_ARGS_NUM);
 double opt_mesh_min_curv_points(OPT_ARGS_NUM);
@@ -485,10 +488,13 @@ double opt_mesh_second_order_experimental(OPT_ARGS_NUM);
 double opt_mesh_second_order_linear(OPT_ARGS_NUM);
 double opt_mesh_second_order_incomplete(OPT_ARGS_NUM);
 double opt_mesh_dual(OPT_ARGS_NUM);
+double opt_mesh_voronoi(OPT_ARGS_NUM);
 double opt_mesh_draw_skin_only(OPT_ARGS_NUM);
 double opt_mesh_save_all(OPT_ARGS_NUM);
+double opt_mesh_save_parametric(OPT_ARGS_NUM);
 double opt_mesh_save_groups_of_nodes(OPT_ARGS_NUM);
 double opt_mesh_color_carousel(OPT_ARGS_NUM);
+double opt_mesh_zone_definition(OPT_ARGS_NUM);
 double opt_mesh_nb_nodes(OPT_ARGS_NUM);
 double opt_mesh_nb_triangles(OPT_ARGS_NUM);
 double opt_mesh_nb_quadrangles(OPT_ARGS_NUM);
@@ -497,6 +503,27 @@ double opt_mesh_nb_hexahedra(OPT_ARGS_NUM);
 double opt_mesh_nb_prisms(OPT_ARGS_NUM);
 double opt_mesh_nb_pyramids(OPT_ARGS_NUM);
 double opt_mesh_cpu_time(OPT_ARGS_NUM);
+double opt_mesh_partition_partitioner(OPT_ARGS_NUM);
+double opt_mesh_partition_num(OPT_ARGS_NUM);
+double opt_mesh_partition_chaco_global_method(OPT_ARGS_NUM);
+double opt_mesh_partition_chaco_architecture(OPT_ARGS_NUM);
+double opt_mesh_partition_chaco_ndims_tot(OPT_ARGS_NUM);
+double opt_mesh_partition_chaco_mesh_dims1(OPT_ARGS_NUM);
+double opt_mesh_partition_chaco_mesh_dims2(OPT_ARGS_NUM);
+double opt_mesh_partition_chaco_mesh_dims3(OPT_ARGS_NUM);
+double opt_mesh_partition_chaco_local_method(OPT_ARGS_NUM);
+double opt_mesh_partition_chaco_eigensolver(OPT_ARGS_NUM);
+double opt_mesh_partition_chaco_vmax(OPT_ARGS_NUM);
+double opt_mesh_partition_chaco_nsection(OPT_ARGS_NUM);
+double opt_mesh_partition_chaco_eigtol(OPT_ARGS_NUM);
+double opt_mesh_partition_chaco_seed(OPT_ARGS_NUM);
+double opt_mesh_partition_chaco_refine_partition(OPT_ARGS_NUM);
+double opt_mesh_partition_chaco_internal_vertices(OPT_ARGS_NUM);
+double opt_mesh_partition_chaco_refine_map(OPT_ARGS_NUM);
+double opt_mesh_partition_chaco_terminal_propogation(OPT_ARGS_NUM);
+double opt_mesh_partition_metis_algorithm(OPT_ARGS_NUM);
+double opt_mesh_partition_metis_edge_matching(OPT_ARGS_NUM);
+double opt_mesh_partition_metis_refine_algorithm(OPT_ARGS_NUM);
 double opt_solver_max_delay(OPT_ARGS_NUM);
 double opt_solver_listen(OPT_ARGS_NUM);
 double opt_solver_plugins(OPT_ARGS_NUM);
@@ -652,6 +679,7 @@ double opt_print_pos_element(OPT_ARGS_NUM);
 double opt_print_pos_gamma(OPT_ARGS_NUM);
 double opt_print_pos_eta(OPT_ARGS_NUM);
 double opt_print_pos_rho(OPT_ARGS_NUM);
+double opt_print_pos_disto(OPT_ARGS_NUM);
 double opt_print_gif_dither(OPT_ARGS_NUM);
 double opt_print_gif_sort(OPT_ARGS_NUM);
 double opt_print_gif_interlace(OPT_ARGS_NUM);
