@@ -60,12 +60,14 @@ std::string SanitizeTeXString(const char *in, int equation)
   return out;
 }
 
-void FixWindowsPath(const char *in, char *out)
+std::string FixWindowsPath(const char *in)
 {
 #if defined(__CYGWIN__)
-  cygwin_conv_to_win32_path(in, out);
+  char tmp[1024];
+  cygwin_conv_to_win32_path(in, tmp);
+  return std::string(tmp);
 #else
-  strcpy(out, in);
+  return std::string(in);
 #endif
 }
 
@@ -88,3 +90,14 @@ void SplitFileName(const char *name, char *no_ext, char *ext, char *base)
     }
   }
 }
+
+std::vector<std::string> SplitWhiteSpace(std::string in, unsigned int len)
+{
+  std::vector<std::string> out(1);
+  for(unsigned int i = 0; i < in.size(); i++){
+    out.back() += in[i];
+    if(out.back().size() > len && in[i] == ' ')
+      out.resize(out.size() + 1);
+  }
+  return out;
+}
