@@ -9,127 +9,88 @@
 
 #include <stdlib.h>
 #include <string.h>
-
 #include "MallocUtils.h"
 #include "TreeUtils.h"
-#include "Message.h"
 
 Tree_T *Tree_Create(int size, int (*fcmp) (const void *a, const void *b))
 {
-  Tree_T *tree;
-
-  tree = (Tree_T *) Malloc(sizeof(Tree_T));
-
+  Tree_T *tree = (Tree_T*)Malloc(sizeof(Tree_T));
   tree->size = size;
   tree->root = avl_init_table(fcmp);
-
-  return (tree);
+  return tree;
 }
 
 void Tree_Delete(Tree_T * tree)
 {
-  if(!tree)
-    return;
+  if(!tree) return;
   avl_free_table(tree->root, Free, 0);
   Free(tree);
 }
 
-void Tree_Add(Tree_T * tree, void *data)
+void *Tree_Add(Tree_T * tree, void *data)
 {
-  void *ptr;
-
-  if(!tree)
-    Msg::Error("Impossible to add in unallocated tree");
-  else {
-    ptr = Malloc(tree->size);
-    memcpy(ptr, data, tree->size);
-    avl_insert(tree->root, ptr, ptr);
-  }
+  if(!tree) return 0;
+  void *ptr = Malloc(tree->size);
+  memcpy(ptr, data, tree->size);
+  avl_insert(tree->root, ptr, ptr);
+  return ptr;
 }
 
 int Tree_Nbr(Tree_T * tree)
 {
-  if(!tree)
-    return 0;
-  return (avl_count(tree->root));
+  if(!tree) return 0;
+  return avl_count(tree->root);
 }
 
 int Tree_Insert(Tree_T * tree, void *data)
 {
-  if(Tree_Search(tree, data) == 0){
+  if(!Tree_Search(tree, data)){
     Tree_Add(tree, data);
     return 1;
   }
-  else{
-    return 0;
-  }
+  return 0;
 }
 
 int Tree_Search(Tree_T * tree, void *data)
 {
+  if(!tree) return 0;
   void *ptr;
-
-  if(!tree)
-    return 0;
-  return (avl_lookup(tree->root, data, &ptr));
+  return avl_lookup(tree->root, data, &ptr);
 }
 
 int Tree_Query(Tree_T * tree, void *data)
 {
+  if(!tree) return 0;
   void *ptr;
-  int state;
-
-  if(!tree)
+  if(!avl_lookup(tree->root, data, &ptr))
     return 0;
-
-  state = avl_lookup(tree->root, data, &ptr);
-
-  if(state == 0)
-    return (0);
-
   memcpy(data, ptr, tree->size);
-
-  return (1);
+  return 1;
 }
 
 void *Tree_PQuery(Tree_T * tree, void *data)
 {
+  if(!tree) return 0;
   void *ptr;
-  int state;
-
-  if(!tree)
+  if(!avl_lookup(tree->root, data, &ptr))
     return 0;
-
-  state = avl_lookup(tree->root, data, &ptr);
-
-  if(state == 0)
-    return (NULL);
-  return (ptr);
+  return ptr;
 }
 
 int Tree_Suppress(Tree_T * tree, void *data)
 {
-  void *ptr;
-  int state;
-
-  if(!tree)
+  if(!tree) return 0;
+  void *ptr = data;
+  if(!avl_delete(tree->root, &ptr, &ptr))
     return 0;
-
-  ptr = data;
-  state = avl_delete(tree->root, &ptr, &ptr);
-  if(state == 0)
-    return (0);
-
   Free(ptr);
-  return (1);
+  return 1;
 }
 
 int Tree_Size(Tree_T * tree)
 {
-  if(!tree)
-    return 0;
-
-  return (tree->size);
+  if(!tree) return 0;
+  return tree->size;
 }
 
 static List_T *pListTransfer;
@@ -147,5 +108,5 @@ List_T *Tree2List(Tree_T * pTree)
     Nb = 1;
   pListTransfer = List_Create(Nb, Nb, Tree_Size(pTree));
   Tree_Action(pTree, TransferList);
-  return (pListTransfer);
+  return pListTransfer;
 }
