@@ -20,6 +20,7 @@ class ExtrudeParams;
 class GEdge : public GEntity {
  private:
   double _length;
+  bool _tooSmall;
 
  protected:
   GVertex *v0, *v1;
@@ -29,6 +30,9 @@ class GEdge : public GEntity {
   GEdge(GModel *model, int tag, GVertex *_v0, GVertex *_v1);
   virtual ~GEdge();
 
+  // delete mesh data
+  virtual void deleteMesh();
+
   // get the start/end vertices of the edge
   GVertex *getBeginVertex() const { return v0; }
   GVertex *getEndVertex() const { return v1; }
@@ -49,10 +53,11 @@ class GEdge : public GEntity {
   // get the bounding box
   virtual SBoundingBox3d bounds() const;
 
-  // faces that bound this entity bounds
+  // faces that this entity bounds
   virtual std::list<GFace*> faces() const { return l_faces; }
 
   // get the parameter location for a point in space on the edge
+  // (returns std::numeric_limits<double>::max() if failed)
   virtual double parFromPoint(const SPoint3 &) const = 0;
 
   // get the point for the given parameter location
@@ -75,13 +80,7 @@ class GEdge : public GEntity {
   virtual double curvature(double par) const;
 
   // reparmaterize the point onto the given face
-  virtual SPoint2 reparamOnFace(GFace *face, double epar,int dir) const;
-
-  // recompute the mesh partitions defined on this edge
-  void recomputeMeshPartitions();
-
-  // delete the mesh partitions defined on this edge
-  void deleteMeshPartitions();
+  virtual SPoint2 reparamOnFace(GFace *face, double epar, int dir) const;
 
   // return the minimum number of segments used for meshing the edge
   virtual int minimumMeshSegments() const { return 1; }
@@ -104,10 +103,21 @@ class GEdge : public GEntity {
   virtual double prescribedMeshSizeAtVertex() const { return meshAttributes.meshSize; }
 
   // true if start == end and no more than 2 segments
-  bool isMeshDegenerated() const{ return (v0 == v1 && mesh_vertices.size() < 2); }
+  void setTooSmall(bool b) { _tooSmall = b; }
+  bool isMeshDegenerated() const 
+  { return _tooSmall || (v0 == v1 && mesh_vertices.size() < 2); }
 
-  // get number of elements in the mesh and get element by index
+  // number of types of elements
+  int getNumElementTypes() const { return 1; }
+
+  // get total/by-type number of elements in the mesh
   unsigned int getNumMeshElements();
+  void getNumMeshElements(unsigned *const c) const;
+
+  // get the start of the array of a type of element
+  MElement *const *getStartElementType(int type) const;
+
+  // get the element at the given index
   MElement *getMeshElement(unsigned int index);
 
   // reset the mesh attributes to default values
