@@ -3,23 +3,24 @@
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
-#include <string.h>
+#include <sstream>
 #include "GModel.h"
 #include "MElement.h"
 #include "discreteRegion.h"
 #include "discreteFace.h"
 #include "discreteEdge.h"
 #include "discreteVertex.h"
+#include "GmshMessage.h"
 
 #if defined(HAVE_GMSH_EMBEDDED)
-#  include "GmshEmbedded.h"
+#include "GmshEmbedded.h"
 #else
-#  include "Message.h"
-#  include "gmshSurface.h"
-#  include "Octree.h"
-#  include "Field.h"
-#  include "Generator.h"
-#  include "Context.h"
+#include "gmshSurface.h"
+#include "Octree.h"
+#include "SmoothData.h"
+#include "Field.h"
+#include "Generator.h"
+#include "Context.h"
 #endif
 
 extern Context_T CTX;
@@ -31,6 +32,7 @@ GModel::GModel(std::string name)
   : _octree(0), _geo_internals(0), _occ_internals(0), _fm_internals(0), 
     _fields(0), _currentMeshEntity(0), modelName(name), normals(0)
 {
+  partitionSize[0] = 0; partitionSize[1] = 0;
   list.push_back(this);
 
 #if !defined(HAVE_GMSH_EMBEDDED)
@@ -97,15 +99,15 @@ void GModel::destroy()
     delete *it;
   vertices.clear();
 
-  if(normals) delete normals;
-  normals = 0;
-
   destroyMeshCaches();
 
   MVertex::resetGlobalNumber();
   MElement::resetGlobalNumber();
 
 #if !defined(HAVE_GMSH_EMBEDDED)
+  if(normals) delete normals;
+  normals = 0;
+
   _fields->reset();
   gmshSurface::reset();
 #endif
@@ -121,6 +123,11 @@ void GModel::destroyMeshCaches()
 #endif
 }
 
+bool GModel::empty() const
+{
+  return vertices.empty() && edges.empty() && faces.empty() && regions.empty();
+}
+
 GRegion *GModel::getRegionByTag(int n) const
 {
   GEntity tmp((GModel*)this, n);
@@ -220,127 +227,74 @@ void GModel::snapVertices()
   }
 }
 
-std::vector<GEntity*> GModel::getEntities()
+void GModel::getEntities(std::vector<GEntity*> &entities)
 {
-  std::vector<GEntity*> entities;
-  for(GModel::viter it = firstVertex(); it != lastVertex(); ++it)
-    entities.push_back(*it);
-  for(GModel::eiter it = firstEdge(); it != lastEdge(); ++it)
-    entities.push_back(*it);
-  for(GModel::fiter it = firstFace(); it != lastFace(); ++it)
-    entities.push_back(*it);
-  for(GModel::riter it = firstRegion(); it != lastRegion(); ++it)
-    entities.push_back(*it);
-  return entities;
+  entities.clear();
+  entities.insert(entities.end(), vertices.begin(), vertices.end());
+  entities.insert(entities.end(), edges.begin(), edges.end());
+  entities.insert(entities.end(), faces.begin(), faces.end());
+  entities.insert(entities.end(), regions.begin(), regions.end());
 }
 
 bool GModel::noPhysicalGroups()
 {
-  for(viter it = firstVertex(); it != lastVertex(); ++it)
-    if((*it)->physicals.size()) return false;
-  for(eiter it = firstEdge(); it != lastEdge(); ++it)
-    if((*it)->physicals.size()) return false;
-  for(fiter it = firstFace(); it != lastFace(); ++it)
-    if((*it)->physicals.size()) return false;
-  for(riter it = firstRegion(); it != lastRegion(); ++it)
-    if((*it)->physicals.size()) return false;
+  std::vector<GEntity*> entities;
+  getEntities(entities);
+  for(unsigned int i = 0; i < entities.size(); i++)
+    if(entities[i]->physicals.size()) return false;
   return true;
 }
 
-static void addInGroup(GEntity *ge, std::map<int, std::vector<GEntity*> > &group)
-{
-  for(unsigned int i = 0; i < ge->physicals.size(); i++){
-    // physicals can be stored with negative signs when the entity
-    // should be "reversed"
-    int p = std::abs(ge->physicals[i]);
-    if(std::find(group[p].begin(), group[p].end(), ge) == group[p].end())
-      group[p].push_back(ge);
-  }
-}
-
 void GModel::getPhysicalGroups(std::map<int, std::vector<GEntity*> > groups[4])
 {
-  for(viter it = firstVertex(); it != lastVertex(); ++it)
-    addInGroup(*it, groups[0]);
-  for(eiter it = firstEdge(); it != lastEdge(); ++it)
-    addInGroup(*it, groups[1]);
-  for(fiter it = firstFace(); it != lastFace(); ++it)
-    addInGroup(*it, groups[2]);
-  for(riter it = firstRegion(); it != lastRegion(); ++it)
-    addInGroup(*it, groups[3]);
+  std::vector<GEntity*> entities;
+  getEntities(entities);
+  for(unsigned int i = 0; i < entities.size(); i++){
+    std::map<int, std::vector<GEntity*> > &group(groups[entities[i]->dim()]);
+    for(unsigned int j = 0; j < entities[i]->physicals.size(); j++){
+      // physicals can be stored with negative signs when the entity
+      // should be "reversed"
+      int p = std::abs(entities[i]->physicals[j]);
+      if(std::find(group[p].begin(), group[p].end(), entities[i]) == group[p].end())
+	group[p].push_back(entities[i]);
+    }
+  }
 }
 
 void GModel::deletePhysicalGroups()
 {
-  for(viter it = firstVertex(); it != lastVertex(); ++it)
-    (*it)->physicals.clear();
-  for(eiter it = firstEdge(); it != lastEdge(); ++it)
-    (*it)->physicals.clear();
-  for(fiter it = firstFace(); it != lastFace(); ++it)
-    (*it)->physicals.clear();
-  for(riter it = firstRegion(); it != lastRegion(); ++it)
-    (*it)->physicals.clear();
+  std::vector<GEntity*> entities;
+  getEntities(entities);
+  for(unsigned int i = 0; i < entities.size(); i++)
+    entities[i]->physicals.clear();
 }
 
 void GModel::deletePhysicalGroup(int dim, int num)
 {
-  switch(dim){
-  case 0:
-    for(viter it = firstVertex(); it != lastVertex(); ++it){
+  std::vector<GEntity*> entities;
+  getEntities(entities);
+  for(unsigned int i = 0; i < entities.size(); i++){
+    if(dim == entities[i]->dim()){
       std::vector<int> p;
-      for(unsigned int i = 0; i < (*it)->physicals.size(); i++)
-        if((*it)->physicals[i] != num) p.push_back((*it)->physicals[i]);
-      (*it)->physicals = p;
+      for(unsigned int j = 0; j < entities[i]->physicals.size(); j++)
+        if(entities[i]->physicals[j] != num) 
+	  p.push_back(entities[i]->physicals[j]);
+      entities[i]->physicals = p;
     }
-    break;
-  case 1:
-    for(eiter it = firstEdge(); it != lastEdge(); ++it){
-      std::vector<int> p;
-      for(unsigned int i = 0; i < (*it)->physicals.size(); i++)
-        if((*it)->physicals[i] != num) p.push_back((*it)->physicals[i]);
-      (*it)->physicals = p;
-    }
-    break;
-  case 2:
-    for(fiter it = firstFace(); it != lastFace(); ++it){
-      std::vector<int> p;
-      for(unsigned int i = 0; i < (*it)->physicals.size(); i++)
-        if((*it)->physicals[i] != num) p.push_back((*it)->physicals[i]);
-      (*it)->physicals = p;
-    }
-    break;
-  case 3:
-    for(riter it = firstRegion(); it != lastRegion(); ++it){
-      std::vector<int> p;
-      for(unsigned int i = 0; i < (*it)->physicals.size(); i++)
-        if((*it)->physicals[i] != num) p.push_back((*it)->physicals[i]);
-      (*it)->physicals = p;
-    }
-    break;
   }
 }
 
 int GModel::getMaxPhysicalNumber()
 {
+  std::vector<GEntity*> entities;
+  getEntities(entities);
   int num = 0;
-  for(viter it = firstVertex(); it != lastVertex(); ++it)
-    for(unsigned int i = 0; i < (*it)->physicals.size(); i++)
-      num = std::max(num, std::abs((*it)->physicals[i]));
-  for(eiter it = firstEdge(); it != lastEdge(); ++it)
-    for(unsigned int i = 0; i < (*it)->physicals.size(); i++)
-      num = std::max(num, std::abs((*it)->physicals[i]));
-  for(fiter it = firstFace(); it != lastFace(); ++it)
-    for(unsigned int i = 0; i < (*it)->physicals.size(); i++)
-      num = std::max(num, std::abs((*it)->physicals[i]));
-  for(riter it = firstRegion(); it != lastRegion(); ++it)
-    for(unsigned int i = 0; i < (*it)->physicals.size(); i++)
-      num = std::max(num, std::abs((*it)->physicals[i]));
+  for(unsigned int i = 0; i < entities.size(); i++)
+    for(unsigned int j = 0; j < entities[i]->physicals.size(); j++)
+      num = std::max(num, std::abs(entities[i]->physicals[j]));
   return num;
 }
 
-
-
-
 int GModel::setPhysicalName(std::string name, int number)
 {
   // check if the name is already used
@@ -362,21 +316,13 @@ std::string GModel::getPhysicalName(int number)
 
 SBoundingBox3d GModel::bounds()
 {
+  std::vector<GEntity*> entities;
+  getEntities(entities);
+  // using the mesh vertices for now; should use entities[i]->bounds() instead
   SBoundingBox3d bb;
-  for(viter it = firstVertex(); it != lastVertex(); ++it)
-    bb += (*it)->bounds();
-
-  // use the mesh instead of the geometry for now
-  for(eiter it = firstEdge(); it != lastEdge(); ++it)
-    for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-      bb += (*it)->mesh_vertices[i]->point();
-  for(fiter it = firstFace(); it != lastFace(); ++it)
-    for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-      bb += (*it)->mesh_vertices[i]->point();
-  for(riter it = firstRegion(); it != lastRegion(); ++it)
-    for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-      bb += (*it)->mesh_vertices[i]->point();
-
+  for(unsigned int i = 0; i < entities.size(); i++)
+    for(unsigned int j = 0; j < entities[i]->mesh_vertices.size(); j++)
+      bb += entities[i]->mesh_vertices[j]->point();
   return bb;
 }
 
@@ -413,30 +359,39 @@ int GModel::getMeshStatus(bool countDiscrete)
 
 int GModel::getNumMeshVertices()
 {
-  int n = 0;
-  for(viter it = firstVertex(); it != lastVertex(); ++it)
-    n += (*it)->mesh_vertices.size();
-  for(eiter it = firstEdge(); it != lastEdge(); ++it)
-    n += (*it)->mesh_vertices.size();
-  for(fiter it = firstFace(); it != lastFace(); ++it)
-    n += (*it)->mesh_vertices.size();
-  for(riter it = firstRegion(); it != lastRegion(); ++it)
-    n += (*it)->mesh_vertices.size();
+  std::vector<GEntity*> entities;
+  getEntities(entities);
+  unsigned int n = 0;
+  for(unsigned int i = 0; i < entities.size(); i++)
+    n += entities[i]->mesh_vertices.size();
   return n;
 }
 
 int GModel::getNumMeshElements()
 {
+  std::vector<GEntity*> entities;
+  getEntities(entities);
   unsigned int n = 0;
-  for(eiter it = firstEdge(); it != lastEdge(); ++it)
-    n += (*it)->getNumMeshElements();
-  for(fiter it = firstFace(); it != lastFace(); ++it)
-    n += (*it)->getNumMeshElements();
-  for(riter it = firstRegion(); it != lastRegion(); ++it)
-    n += (*it)->getNumMeshElements();
+  for(unsigned int i = 0; i < entities.size(); i++)
+    n += entities[i]->getNumMeshElements();
   return n;
 }
 
+int GModel::getNumMeshElements(unsigned c[4])
+{
+  c[0] = 0; c[1] = 0; c[2] = 0; c[3] = 0;
+  for(riter it = firstRegion(); it != lastRegion(); ++it)
+    (*it)->getNumMeshElements(c);
+  if(c[0] + c[1] + c[2] + c[3]) return 3;
+  for(fiter it = firstFace(); it != lastFace(); ++it)
+    (*it)->getNumMeshElements(c);
+  if(c[0] + c[1]) return 2;
+  for(eiter it = firstEdge(); it != lastEdge(); ++it)
+    (*it)->getNumMeshElements(c);
+  if(c[0]) return 1;
+  return 0;
+}
+
 static void MElementBB(void *a, double *min, double *max)
 {
   MElement *e = (MElement*)a;
@@ -489,10 +444,11 @@ MElement *GModel::getMeshElementByCoord(SPoint3 &p)
     const int maxElePerBucket = 100; // memory vs. speed trade-off
     _octree = Octree_Create(maxElePerBucket, min, size, 
 			    MElementBB, MElementCentroid, MElementInEle);
-    std::vector<GEntity*> ent = getEntities();
-    for(unsigned int i = 0; i < ent.size(); i++)
-      for(unsigned int j = 0; j < ent[i]->getNumMeshElements(); j++)
-	Octree_Insert(ent[i]->getMeshElement(j), _octree);
+    std::vector<GEntity*> entities;
+    getEntities(entities);
+    for(unsigned int i = 0; i < entities.size(); i++)
+      for(unsigned int j = 0; j < entities[i]->getNumMeshElements(); j++)
+	Octree_Insert(entities[i]->getMeshElement(j), _octree);
     Octree_Arrange(_octree);
   }
   double P[3] = {p.x(), p.y(), p.z()};
@@ -503,13 +459,6 @@ MElement *GModel::getMeshElementByCoord(SPoint3 &p)
 #endif
 }
 
-template <class T>
-static void insertMeshVertices(std::vector<MVertex*> &vertices, T &container)
-{
-  for(unsigned int i = 0; i < vertices.size(); i++)
-    container[vertices[i]->getNum()] = vertices[i];
-}
-
 MVertex *GModel::getMeshVertexByTag(int n)
 {
   if(_vertexVectorCache.empty() && _vertexMapCache.empty()){
@@ -517,30 +466,25 @@ MVertex *GModel::getMeshVertexByTag(int n)
     _vertexVectorCache.clear();
     _vertexMapCache.clear();
     bool dense = (getNumMeshVertices() == MVertex::getGlobalNumber());
+    std::vector<GEntity*> entities;
+    getEntities(entities);
     if(dense){
+      Msg::Debug("Good: we have a dense vertex numbering in the cache");
       // numbering starts at 1
       _vertexVectorCache.resize(MVertex::getGlobalNumber() + 1);
-      for(viter it = firstVertex(); it != lastVertex(); ++it)
-        insertMeshVertices((*it)->mesh_vertices, _vertexVectorCache);
-      for(eiter it = firstEdge(); it != lastEdge(); ++it)
-        insertMeshVertices((*it)->mesh_vertices, _vertexVectorCache);
-      for(fiter it = firstFace(); it != lastFace(); ++it)
-        insertMeshVertices((*it)->mesh_vertices, _vertexVectorCache);
-      for(riter it = firstRegion(); it != lastRegion(); ++it)
-        insertMeshVertices((*it)->mesh_vertices, _vertexVectorCache);
+      for(unsigned int i = 0; i < entities.size(); i++)
+	for(unsigned int j = 0; j < entities[i]->mesh_vertices.size(); j++)
+	  _vertexVectorCache[entities[i]->mesh_vertices[j]->getNum()] = 
+	    entities[i]->mesh_vertices[j];
     }
     else{
-      for(viter it = firstVertex(); it != lastVertex(); ++it)
-        insertMeshVertices((*it)->mesh_vertices, _vertexMapCache);
-      for(eiter it = firstEdge(); it != lastEdge(); ++it)
-        insertMeshVertices((*it)->mesh_vertices, _vertexMapCache);
-      for(fiter it = firstFace(); it != lastFace(); ++it)
-        insertMeshVertices((*it)->mesh_vertices, _vertexMapCache);
-      for(riter it = firstRegion(); it != lastRegion(); ++it)
-        insertMeshVertices((*it)->mesh_vertices, _vertexMapCache);
+      for(unsigned int i = 0; i < entities.size(); i++)
+	for(unsigned int j = 0; j < entities[i]->mesh_vertices.size(); j++)
+	  _vertexMapCache[entities[i]->mesh_vertices[j]->getNum()] = 
+	    entities[i]->mesh_vertices[j];
     }
   }
-
+  
   if(n < (int)_vertexVectorCache.size())
     return _vertexVectorCache[n];
   else
@@ -611,84 +555,37 @@ void GModel::removeInvisibleElements()
 
 int GModel::indexMeshVertices(bool all)
 {
+  std::vector<GEntity*> entities;
+  getEntities(entities);
+
   // tag all mesh vertices with -1 (negative vertices will not be
   // saved)
-  for(viter it = firstVertex(); it != lastVertex(); ++it)
-    for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-      (*it)->mesh_vertices[i]->setIndex(-1);
-  for(eiter it = firstEdge(); it != lastEdge(); ++it)
-    for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-      (*it)->mesh_vertices[i]->setIndex(-1);
-  for(fiter it = firstFace(); it != lastFace(); ++it)
-    for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-      (*it)->mesh_vertices[i]->setIndex(-1);
-  for(riter it = firstRegion(); it != lastRegion(); ++it)
-    for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-      (*it)->mesh_vertices[i]->setIndex(-1);
-
+  for(unsigned int i = 0; i < entities.size(); i++)
+    for(unsigned int j = 0; j < entities[i]->mesh_vertices.size(); j++)
+      entities[i]->mesh_vertices[j]->setIndex(-1);
+  
   // tag all mesh vertices belonging to elements that need to be saved
   // with 0
-  for(viter it = firstVertex(); it != lastVertex(); ++it)
-    if(all || (*it)->physicals.size()){
-      for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-        (*it)->mesh_vertices[i]->setIndex(0);
-    }
-  for(eiter it = firstEdge(); it != lastEdge(); ++it)
-    if(all || (*it)->physicals.size()){
-      for(unsigned int i = 0; i < (*it)->lines.size(); i++)
-        for(int j = 0; j < (*it)->lines[i]->getNumVertices(); j++)
-          (*it)->lines[i]->getVertex(j)->setIndex(0);
-    }
-  for(fiter it = firstFace(); it != lastFace(); ++it)
-    if(all || (*it)->physicals.size()){
-      for(unsigned int i = 0; i < (*it)->triangles.size(); i++)
-        for(int j = 0; j < (*it)->triangles[i]->getNumVertices(); j++)
-          (*it)->triangles[i]->getVertex(j)->setIndex(0);
-      for(unsigned int i = 0; i < (*it)->quadrangles.size(); i++)
-        for(int j = 0; j < (*it)->quadrangles[i]->getNumVertices(); j++)
-          (*it)->quadrangles[i]->getVertex(j)->setIndex(0);
-    }
-  for(riter it = firstRegion(); it != lastRegion(); ++it)
-    if(all || (*it)->physicals.size()){
-      for(unsigned int i = 0; i < (*it)->tetrahedra.size(); i++)
-        for(int j = 0; j < (*it)->tetrahedra[i]->getNumVertices(); j++)
-          (*it)->tetrahedra[i]->getVertex(j)->setIndex(0);
-      for(unsigned int i = 0; i < (*it)->hexahedra.size(); i++)
-        for(int j = 0; j < (*it)->hexahedra[i]->getNumVertices(); j++)
-          (*it)->hexahedra[i]->getVertex(j)->setIndex(0);
-      for(unsigned int i = 0; i < (*it)->prisms.size(); i++)
-        for(int j = 0; j < (*it)->prisms[i]->getNumVertices(); j++)
-          (*it)->prisms[i]->getVertex(j)->setIndex(0);
-      for(unsigned int i = 0; i < (*it)->pyramids.size(); i++)
-        for(int j = 0; j < (*it)->pyramids[i]->getNumVertices(); j++)
-          (*it)->pyramids[i]->getVertex(j)->setIndex(0);
-    }
+  for(unsigned int i = 0; i < entities.size(); i++)
+    if(all || entities[i]->physicals.size())
+      for(unsigned int j = 0; j < entities[i]->getNumMeshElements(); j++)
+	for(int k = 0; k < entities[i]->getMeshElement(j)->getNumVertices(); k++)
+	  entities[i]->getMeshElement(j)->getVertex(k)->setIndex(0);
 
   // renumber all the mesh vertices tagged with 0
   int numVertices = 0;
-  for(viter it = firstVertex(); it != lastVertex(); ++it)
-    for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-      if(!(*it)->mesh_vertices[i]->getIndex())
-        (*it)->mesh_vertices[i]->setIndex(++numVertices);
-  for(eiter it = firstEdge(); it != lastEdge(); ++it)
-    for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-      if(!(*it)->mesh_vertices[i]->getIndex())
-        (*it)->mesh_vertices[i]->setIndex(++numVertices);
-  for(fiter it = firstFace(); it != lastFace(); ++it)
-    for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-      if(!(*it)->mesh_vertices[i]->getIndex())
-        (*it)->mesh_vertices[i]->setIndex(++numVertices);
-  for(riter it = firstRegion(); it != lastRegion(); ++it)
-    for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-      if(!(*it)->mesh_vertices[i]->getIndex())
-        (*it)->mesh_vertices[i]->setIndex(++numVertices);
-
+  for(unsigned int i = 0; i < entities.size(); i++)
+    for(unsigned int j = 0; j < entities[i]->mesh_vertices.size(); j++)
+      if(!entities[i]->mesh_vertices[j]->getIndex())
+        entities[i]->mesh_vertices[j]->setIndex(++numVertices);
+  
   return numVertices;
 }
 
 void GModel::scaleMesh(double factor)
 {
-  std::vector<GEntity*> entities = getEntities();
+  std::vector<GEntity*> entities;
+  getEntities(entities);
   for(unsigned int i = 0; i < entities.size(); i++)
     for(unsigned int j = 0; j < entities[i]->mesh_vertices.size(); j++){
       MVertex *v = entities[i]->mesh_vertices[j];
@@ -698,134 +595,29 @@ void GModel::scaleMesh(double factor)
     }
 }
 
-std::set<int> &GModel::recomputeMeshPartitions()
+void GModel::recomputeMeshPartitions()
 {
-  for(eiter it = firstEdge(); it != lastEdge(); ++it)
-    (*it)->recomputeMeshPartitions();
-  for(fiter it = firstFace(); it != lastFace(); ++it)
-    (*it)->recomputeMeshPartitions();
-  for(riter it = firstRegion(); it != lastRegion(); ++it)
-    (*it)->recomputeMeshPartitions();
-  return meshPartitions;
+  meshPartitions.clear();
+  std::vector<GEntity*> entities;
+  getEntities(entities);
+  for(unsigned int i = 0; i < entities.size(); i++){
+    for(unsigned int j = 0; j < entities[i]->getNumMeshElements(); j++){
+      int part = entities[i]->getMeshElement(j)->getPartition();
+      if(part) meshPartitions.insert(part);
+    }
+  }
 }
 
 void GModel::deleteMeshPartitions()
 {
-  for(eiter it = firstEdge(); it != lastEdge(); ++it)
-    (*it)->deleteMeshPartitions();
-  for(fiter it = firstFace(); it != lastFace(); ++it)
-    (*it)->deleteMeshPartitions();
-  for(riter it = firstRegion(); it != lastRegion(); ++it)
-    (*it)->deleteMeshPartitions();
+  std::vector<GEntity*> entities;
+  getEntities(entities);
+  for(unsigned int i = 0; i < entities.size(); i++)
+    for(unsigned int j = 0; j < entities[i]->getNumMeshElements(); j++)
+      entities[i]->getMeshElement(j)->setPartition(0);
   meshPartitions.clear();
 }
 
-static int checkVertices(std::vector<MVertex*> &vertices,
-                         std::set<MVertex*, MVertexLessThanLexicographic> &pos)
-{
-  int num = 0;
-  for(unsigned int i = 0; i < vertices.size(); i++){
-    MVertex *v = vertices[i];
-    std::set<MVertex*, MVertexLessThanLexicographic>::iterator it = pos.find(v);
-    if(it == pos.end()){
-      pos.insert(v);
-    }
-    else{
-      Msg::Info("Vertices %d and %d have identical position (%g, %g, %g)",
-          (*it)->getNum(), v->getNum(), v->x(), v->y(), v->z());
-      num++;
-    }
-  }
-  return num;
-}
-
-template <class T>
-static int checkElements(int tag,
-                         std::vector<T*> &elements,
-                         std::set<MElement*, MElementLessThanLexicographic> &pos)
-{
-  int num = 0;
-  for(unsigned int i = 0; i < elements.size(); i++){
-    MElement *e = elements[i];
-    std::set<MElement*, MElementLessThanLexicographic>::iterator it = pos.find(e);
-    if(it == pos.end()){
-      pos.insert(e);
-    }
-    else{
-      char temp[256], temp2[256];
-      sprintf(temp, "Elements %d ( ", (*it)->getNum());
-      for (int i = 0; i < (*it)->getNumVertices();i++){
-        sprintf(temp2, "%d ", (*it)->getVertex(i)->getNum());
-        strcat(temp, temp2);
-      }
-      sprintf(temp2, ") on entity %d has same barycenter as element %d ( ",
-              tag, e->getNum());
-      strcat(temp, temp2);
-      for (int i = 0; i < e->getNumVertices(); i++){
-        sprintf(temp2, "%d ", e->getVertex(i)->getNum());
-        strcat(temp, temp2);
-      }
-      sprintf(temp2, ")");
-      strcat(temp, temp2);
-      Msg::Info("%s", temp);
-      num++;
-    }
-  }
-  return num;
-}
-
-void GModel::checkMeshCoherence()
-{
-  int numEle = getNumMeshElements();
-  if(!numEle) return;
-
-  Msg::Info("Checking mesh coherence (%d elements)", numEle);
-
-  SBoundingBox3d bb = bounds();
-  double lc = bb.empty() ? 1. : norm(SVector3(bb.max(), bb.min()));
-  double tol = CTX.geom.tolerance * lc;
-
-  // check for duplicate mesh vertices
-  {
-    double old_tol = MVertexLessThanLexicographic::tolerance;
-    MVertexLessThanLexicographic::tolerance = tol;
-    std::set<MVertex*, MVertexLessThanLexicographic> pos;
-    int num = 0;
-    for(viter it = firstVertex(); it != lastVertex(); ++it)
-      num += checkVertices((*it)->mesh_vertices, pos);
-    for(eiter it = firstEdge(); it != lastEdge(); ++it)
-      num += checkVertices((*it)->mesh_vertices, pos);
-    for(fiter it = firstFace(); it != lastFace(); ++it)
-      num += checkVertices((*it)->mesh_vertices, pos);
-    for(riter it = firstRegion(); it != lastRegion(); ++it)
-      num += checkVertices((*it)->mesh_vertices, pos);
-    if(num) Msg::Warning("%d duplicate vertices", num);
-    MVertexLessThanLexicographic::tolerance = old_tol;
-  }
-
-  // check for duplicate elements
-  {
-    double old_tol = MElementLessThanLexicographic::tolerance;
-    MElementLessThanLexicographic::tolerance = tol;
-    std::set<MElement*, MElementLessThanLexicographic> pos;
-    int num = 0;
-    for(eiter it = firstEdge(); it != lastEdge(); ++it)
-      num += checkElements((*it)->tag(), (*it)->lines, pos);
-    for(fiter it = firstFace(); it != lastFace(); ++it){
-      num += checkElements((*it)->tag(), (*it)->triangles, pos);
-      num += checkElements((*it)->tag(), (*it)->quadrangles, pos);
-    }
-    for(riter it = firstRegion(); it != lastRegion(); ++it){
-      num += checkElements((*it)->tag(), (*it)->tetrahedra, pos);
-      num += checkElements((*it)->tag(), (*it)->hexahedra, pos);
-      num += checkElements((*it)->tag(), (*it)->prisms, pos);
-      num += checkElements((*it)->tag(), (*it)->pyramids, pos);
-    }
-    if(num) Msg::Warning("%d duplicate elements", num);
-    MElementLessThanLexicographic::tolerance = old_tol;
-  }
-}
-
 template<class T>
 static void _addElements(std::vector<T*> &dst, const std::vector<MElement*> &src)
 {
@@ -839,6 +631,17 @@ void GModel::_storeElementsInEntities(std::map<int, std::vector<MElement*> > &ma
     if(!it->second.size()) continue;
     int numEdges = it->second[0]->getNumEdges();
     switch(numEdges){
+    case 0: 
+      {
+        GVertex *v = getVertexByTag(it->first);
+        if(!v){
+          v = new discreteVertex(this, it->first);
+          add(v);
+        }
+	if(v->points.empty()) // CAD points already have one by default
+	  _addElements(v->points, it->second);
+      }
+      break;
     case 1: 
       {
         GEdge *e = getEdgeByTag(it->first);
@@ -881,8 +684,12 @@ template<class T>
 static void _associateEntityWithElementVertices(GEntity *ge, std::vector<T*> &elements)
 {
   for(unsigned int i = 0; i < elements.size(); i++)
-    for(int j = 0; j < elements[i]->getNumVertices(); j++)
-      elements[i]->getVertex(j)->setEntity(ge);
+    for(int j = 0; j < elements[i]->getNumVertices(); j++){
+      if (!elements[i]->getVertex(j)->onWhat() ||
+	  elements[i]->getVertex(j)->onWhat()->dim() > ge->dim()){
+	elements[i]->getVertex(j)->setEntity(ge);
+      }
+    }
 }
 
 void GModel::_associateEntityWithMeshVertices()
@@ -905,7 +712,190 @@ void GModel::_associateEntityWithMeshVertices()
     _associateEntityWithElementVertices(*it, (*it)->lines);
   }
   for(viter it = firstVertex(); it != lastVertex(); ++it){
-    (*it)->mesh_vertices[0]->setEntity(*it);
+    _associateEntityWithElementVertices(*it, (*it)->points);
   }
 }
 
+void GModel::_storeVerticesInEntities(std::map<int, MVertex*> &vertices)
+{
+  std::map<int, MVertex*>::const_iterator it = vertices.begin();
+  for(; it != vertices.end(); ++it){
+    MVertex *v = it->second;
+    GEntity *ge = v->onWhat();
+    if(ge){
+      if(ge->dim() || ge->mesh_vertices.empty()){ // special case for points
+	ge->mesh_vertices.push_back(v);
+      }
+    }
+    else
+      delete v; // we delete all unused vertices
+  }
+}
+
+void GModel::_storeVerticesInEntities(std::vector<MVertex*> &vertices)
+{
+  for(unsigned int i = 0; i < vertices.size(); i++){
+    MVertex *v = vertices[i];
+    if(v){ // the vector is allowed to have null entries
+      GEntity *ge = v->onWhat();
+      if(ge) {
+	if(ge->dim() || ge->mesh_vertices.empty()){ // special case for points
+	  ge->mesh_vertices.push_back(v);
+	}
+      }
+      else
+        delete v; // we delete all unused vertices
+    }
+  }
+}
+
+void GModel::checkMeshCoherence(double tolerance)
+{
+  int numEle = getNumMeshElements();
+  if(!numEle) return;
+
+  Msg::Info("Checking mesh coherence (%d elements)", numEle);
+
+  SBoundingBox3d bbox = bounds();
+  double lc = bbox.empty() ? 1. : norm(SVector3(bbox.max(), bbox.min()));
+
+  std::vector<GEntity*> entities;
+  getEntities(entities);
+
+  // check for duplicate mesh vertices
+  {
+    double old_tol = MVertexLessThanLexicographic::tolerance;
+    MVertexLessThanLexicographic::tolerance = tolerance * lc;
+    std::set<MVertex*, MVertexLessThanLexicographic> pos;
+    int num = 0;
+    for(unsigned int i = 0; i < entities.size(); i++){
+      for(unsigned int j = 0; j < entities[i]->mesh_vertices.size(); j++){
+	MVertex *v = entities[i]->mesh_vertices[j];
+	std::set<MVertex*, MVertexLessThanLexicographic>::iterator it = pos.find(v);
+	if(it == pos.end()){
+	  pos.insert(v);
+	}
+	else{
+	  Msg::Info("Vertices %d and %d have identical position (%g, %g, %g)",
+		    (*it)->getNum(), v->getNum(), v->x(), v->y(), v->z());
+	  num++;
+	}
+      }
+    }
+    if(num) Msg::Warning("%d duplicate vertices", num);
+    MVertexLessThanLexicographic::tolerance = old_tol;
+  }
+
+  // check for duplicate elements
+  {
+    double old_tol = MElementLessThanLexicographic::tolerance;
+    MElementLessThanLexicographic::tolerance = tolerance * lc;
+    std::set<MElement*, MElementLessThanLexicographic> pos;
+    int num = 0;
+    for(unsigned int i = 0; i < entities.size(); i++){
+      for(unsigned int j = 0; j < entities[i]->getNumMeshElements(); j++){
+	MElement *e = entities[i]->getMeshElement(j);
+	std::set<MElement*, MElementLessThanLexicographic>::iterator it = pos.find(e);
+	if(it == pos.end()){
+	  pos.insert(e);
+	}
+	else{
+	  std::ostringstream sstream;
+	  sstream << "Element " << e->getNum() << " [ ";
+	  for (int k = 0; k < e->getNumVertices(); k++)
+	    sstream << e->getVertex(k)->getNum() << " ";
+	  sstream << "] on entity " << entities[i]->tag() 
+		  << " has same barycenter as element " << (*it)->getNum() 
+		  << " [ ";
+	  for (int k = 0; k < (*it)->getNumVertices(); k++)
+	    sstream << (*it)->getVertex(k)->getNum() << " ";
+	  sstream << "]";
+	  Msg::Info("%s", sstream.str().c_str());
+	  num++;
+	}
+      }
+    }
+    if(num) Msg::Warning("%d duplicate elements", num);
+    MElementLessThanLexicographic::tolerance = old_tol;
+  }
+}
+
+int GModel::removeDuplicateMeshVertices(double tolerance)
+{
+  SBoundingBox3d bbox = bounds();
+  double lc = bbox.empty() ? 1. : norm(SVector3(bbox.max(), bbox.min()));
+
+  std::vector<GEntity*> entities;
+  getEntities(entities);
+
+  double old_tol = MVertexLessThanLexicographic::tolerance;
+  MVertexLessThanLexicographic::tolerance = tolerance * lc;
+  std::set<MVertex*, MVertexLessThanLexicographic> pos;
+
+  for(unsigned int i = 0; i < entities.size(); i++){
+    for(unsigned int j = 0; j < entities[i]->mesh_vertices.size(); j++){
+      MVertex *v = entities[i]->mesh_vertices[j];
+      MVertex w(v->x(), v->y(), v->z());
+      std::set<MVertex*, MVertexLessThanLexicographic>::iterator it = pos.find(&w);
+      if(it == pos.end())
+        pos.insert(new MVertex(v->x(), v->y(), v->z()));
+    }
+  }
+
+  int diff = getNumMeshVertices() - pos.size();
+  if(!diff){
+    for(std::set<MVertex*, MVertexLessThanLexicographic>::iterator it = pos.begin();
+        it != pos.end(); it++)
+      delete *it;
+    Msg::Info("No duplicate vertices found");
+    return 0;
+  }
+
+  std::map<int, std::vector<MElement*> > elements[8];
+  for(unsigned int i = 0; i < entities.size(); i++){
+    for(unsigned int j = 0; j < entities[i]->getNumMeshElements(); j++){
+      MElement *e = entities[i]->getMeshElement(j);
+      std::vector<MVertex*> verts;
+      for(int k = 0; k < e->getNumVertices(); k++){
+        MVertex *v = e->getVertex(k);
+        std::set<MVertex*, MVertexLessThanLexicographic>::iterator it = pos.find(v);
+        if(it != pos.end())
+          verts.push_back(*it);
+        else
+          Msg::Error("Could not find unique vertex (%g,%g,%g)", v->x(), v->y(), v->z());
+      }
+      MElementFactory factory;
+      MElement *e2 = factory.create(e->getTypeForMSH(), verts, e->getNum(), 
+                                    e->getPartition());
+      switch(e2->getNumEdges()){
+      case 0: elements[0][entities[i]->tag()].push_back(e2); break;
+      case 1: elements[1][entities[i]->tag()].push_back(e2); break;
+      case 3: elements[2][entities[i]->tag()].push_back(e2); break;
+      case 4: elements[3][entities[i]->tag()].push_back(e2); break;
+      case 6: elements[4][entities[i]->tag()].push_back(e2); break;
+      case 12: elements[5][entities[i]->tag()].push_back(e2); break;
+      case 9: elements[6][entities[i]->tag()].push_back(e2); break;
+      case 8: elements[7][entities[i]->tag()].push_back(e2); break;
+      }
+    }
+  }
+
+  for(unsigned int i = 0; i < entities.size(); i++)
+    entities[i]->deleteMesh();
+
+  std::vector<MVertex*> vertices;
+  for(std::set<MVertex*, MVertexLessThanLexicographic>::iterator it = pos.begin();
+      it != pos.end(); it++)
+    vertices.push_back(*it);
+
+  for(int i = 0; i < (int)(sizeof(elements) / sizeof(elements[0])); i++) 
+    _storeElementsInEntities(elements[i]);
+  _associateEntityWithMeshVertices();
+  _storeVerticesInEntities(vertices);
+
+  MVertexLessThanLexicographic::tolerance = old_tol;
+
+  Msg::Info("Removed %d duplicate mesh vertices", diff);
+    
+  return diff;
+}
