@@ -3,17 +3,12 @@
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
-#include <string.h>
+#include <sstream>
 #include <algorithm>
 #include "GVertex.h"
 #include "GFace.h"
-#include "MVertex.h"
-
-#if defined(HAVE_GMSH_EMBEDDED)
-#  include "GmshEmbedded.h"
-#else
-#  include "Message.h"
-#endif
+#include "MElement.h"
+#include "GmshMessage.h"
 
 GVertex::GVertex(GModel *m, int tag, double ms) : GEntity(m, tag), meshSize(ms) 
 {
@@ -21,8 +16,15 @@ GVertex::GVertex(GModel *m, int tag, double ms) : GEntity(m, tag), meshSize(ms)
 
 GVertex::~GVertex()
 {
-  for(unsigned int i = 0; i < mesh_vertices.size(); i++)
-    delete mesh_vertices[i];
+  deleteMesh();
+}
+
+void GVertex::deleteMesh()
+{
+  for(unsigned int i = 0; i < mesh_vertices.size(); i++) delete mesh_vertices[i];
+  mesh_vertices.clear();
+  for(unsigned int i = 0; i < points.size(); i++) delete points[i];
+  points.clear();
 }
 
 void GVertex::setPosition(GPoint &p)
@@ -47,13 +49,21 @@ SPoint2 GVertex::reparamOnFace(GFace *gf, int) const
 
 std::string GVertex::getAdditionalInfoString()
 {
-  char str[256];
-  sprintf(str, "{%g,%g,%g}", x(), y(), z());
+  std::ostringstream sstream;
+  sstream << "{" << x() << "," << y() << "," << z() << "}";
   double lc = prescribedMeshSizeAtVertex();
-  if(lc < 1.e22){
-    char str2[256];
-    sprintf(str2, " (cl: %g)", lc);
-    strcat(str, str2);
-  }
-  return std::string(str);
+  if(lc < MAX_LC) sstream << " (cl: " << lc << ")";
+  return sstream.str();
+}
+
+unsigned int GVertex::getNumMeshElements()
+{
+  return points.size(); 
+}
+
+MElement *GVertex::getMeshElement(unsigned int index)
+{ 
+  if(index < points.size())
+    return points[index]; 
+  return 0;
 }
