@@ -4,7 +4,7 @@
 // bugs and problems to <gmsh@geuz.org>.
 
 #include <string.h>
-#include "Message.h"
+#include "GmshMessage.h"
 #include "Numeric.h"
 #include "MallocUtils.h"
 #include "Geo.h"
@@ -117,8 +117,8 @@ Vertex *Create_Vertex(int Num, double X, double Y, double Z, double lc, double u
 
 Vertex *Create_Vertex(int Num, double u, double v, gmshSurface *surf, double lc)
 {
-  SPoint3 p = surf->point(u,v);
-  Vertex *pV = new Vertex(p.x(),p.y(),p.z(),lc);
+  SPoint3 p = surf->point(u, v);
+  Vertex *pV = new Vertex(p.x(), p.y(), p.z(), lc);
   pV->w = 1.0;
   pV->Num = Num;
   GModel::current()->getGEOInternals()->MaxPointNum = 
@@ -138,7 +138,7 @@ static void Free_Vertex(void *a, void *b)
   }
 }
 
-PhysicalGroup *Create_PhysicalGroup(int Num, int typ, List_T *intlist)
+PhysicalGroup *Create_PhysicalGroup(int Num, int typ, List_T *intlist, List_T *bndlist[4])
 {
   PhysicalGroup *p = (PhysicalGroup *)Malloc(sizeof(PhysicalGroup));
   p->Entities = List_Create(List_Nbr(intlist), 1, sizeof(int));
@@ -152,6 +152,17 @@ PhysicalGroup *Create_PhysicalGroup(int Num, int typ, List_T *intlist)
     List_Read(intlist, i, &j);
     List_Add(p->Entities, &j);
   }
+  p->Boundaries[0] = p->Boundaries[1] = p->Boundaries[2] = p->Boundaries[3] = 0;
+  if (bndlist){
+    for(int i = 0; i < 4; i++) {
+      p->Boundaries[i] = List_Create(List_Nbr(bndlist[i]), 1, sizeof(int));
+      for(int j = 0; j < List_Nbr(bndlist[i]); j++) {
+	int k;
+	List_Read(bndlist[i], j, &k);
+	List_Add(p->Boundaries[i], &k);
+      }
+    }
+  }
   return p;
 }
 
@@ -576,7 +587,7 @@ Surface *Create_Surface(int Num, int Typ)
   pS->Visible = 1;
   pS->Num = Num;
   pS->geometry = 0;
-  pS->RuledSurfaceOptions = 0;
+  pS->InSphereCenter = 0;
 
   GModel::current()->getGEOInternals()->MaxSurfaceNum = 
     std::max(GModel::current()->getGEOInternals()->MaxSurfaceNum, Num);
@@ -2231,7 +2242,10 @@ static int Extrude_ProtudeSurface(int type, int is,
 
   *pv = NULL;
 
-  if(!(ps = FindSurface(is)))
+  // 'is' can be negative, to signify that the surface orientation
+  // should be reverted. This orientation information is only used at
+  // the moment when creating boundary layers
+  if(!(ps = FindSurface(std::abs(is))))
     return 0;
 
   Msg::Debug("Extrude Surface %d", is);
@@ -2240,7 +2254,7 @@ static int Extrude_ProtudeSurface(int type, int is,
 
   chapeau->Extrude = new ExtrudeParams(COPIED_ENTITY);
   chapeau->Extrude->fill(type, T0, T1, T2, A0, A1, A2, X0, X1, X2, alpha);
-  chapeau->Extrude->geo.Source = ps->Num;
+  chapeau->Extrude->geo.Source = is; // not ps->Num: we need the sign info
   if(e)
     chapeau->Extrude->mesh = e->mesh;
 
@@ -2254,7 +2268,8 @@ static int Extrude_ProtudeSurface(int type, int is,
       }
     c->Extrude = new ExtrudeParams(COPIED_ENTITY);
     c->Extrude->fill(type, T0, T1, T2, A0, A1, A2, X0, X1, X2, alpha);
-    //pas de abs()! il faut le signe pour copy_mesh dans ExtrudeMesh
+    // don't take the abs(): the sign of c2->Num is important (used
+    // when copying the mesh in the extrusion routine)
     c->Extrude->geo.Source = c2->Num;
     if(e)
       c->Extrude->mesh = e->mesh;
@@ -2870,11 +2885,9 @@ static Vertex *VERTEX;
 
 static double min1d(double (*funct) (double), double *xmin)
 {
-  // we should think about the tolerance more carefully...
-  double ax = 1.e-15, bx = 1.e-12, cx = 1.e-11, fa, fb, fx, tol = 1.e-4;
-  mnbrak(&ax, &bx, &cx, &fa, &fx, &fb, funct);
-  //Msg::Info("--MIN1D : ax %12.5E bx %12.5E cx %12.5E",ax,bx,cx);  
-  return (brent(ax, bx, cx, funct, tol, xmin));
+  // 0. for tolerance allows for maximum as code in gsl_brent
+  return (brent(CURVE->ubeg, 0.5*(CURVE->ubeg + CURVE->uend), CURVE->uend,
+                funct, 0., xmin));
 }
 
 static void projectPS(int N, double x[], double res[])
@@ -2896,10 +2909,6 @@ static void projectPS(int N, double x[], double res[])
 
 static double projectPC(double u)
 {
-  if(u < CURVE->ubeg)
-    u = CURVE->ubeg;
-  if(u < CURVE->ubeg)
-    u = CURVE->ubeg;
   Vertex c = InterpolateCurve(CURVE, u, 0);
   return sqrt(SQU(c.Pos.X - VERTEX->Pos.X) +
               SQU(c.Pos.Y - VERTEX->Pos.Y) + 
@@ -2950,6 +2959,117 @@ bool ProjectPointOnSurface(Surface *s, Vertex &p, double u[2])
   return true;
 }
 
+// Split line
+
+static Curve *_create_splitted_curve(Curve *c,List_T *nodes)
+{
+  int  beg, end;
+  List_Read(nodes, 0, &beg);
+  List_Read(nodes, List_Nbr(nodes) - 1, &end);
+  int id = NEWLINE();
+  Curve *cnew = NULL;
+  switch(c->Typ){
+  case MSH_SEGM_LINE:
+    cnew = Create_Curve(id, c->Typ, 1, nodes, NULL, -1, -1, 0., 1.);
+    break;
+  case MSH_SEGM_SPLN:
+    cnew = Create_Curve(id, c->Typ, 3, nodes, NULL, -1, -1, 0., 1.);
+    break;
+  case MSH_SEGM_BSPLN:
+    cnew = Create_Curve(id, c->Typ, 2, nodes, NULL, -1, -1, 0., 1.);
+    break;
+  default : //should never reach this point...
+    Msg::Error("Cannot split a curve with type %i", c->Typ);
+    return NULL;
+  }
+  Tree_Add(GModel::current()->getGEOInternals()->Curves, &cnew);
+  CreateReversedCurve(cnew);
+  return cnew;
+}
+
+bool SplitCurve(int line_id, List_T *vertices_id, List_T *shapes)
+{
+  Curve *c = FindCurve(line_id);
+  if(!c){
+    Msg::Error("Curve %i does not exists", line_id);
+    return false;
+  }
+  switch (c->Typ){
+  case MSH_SEGM_LINE:
+  case MSH_SEGM_SPLN:
+  case MSH_SEGM_BSPLN:
+    break;
+  default:
+    Msg::Error("Cannot split curve %i with type %i", line_id, c->Typ);
+    return false;
+  }
+  std::set<int>v_break;
+  for(int i = 0; i < List_Nbr(vertices_id); i++){
+    int id;
+    List_Read(vertices_id, i, &id);
+    v_break.insert(id);
+  }
+  bool is_periodic = (c->beg == c->end);
+  bool first_periodic = true;
+  bool last_periodic = false;
+  List_T *new_list = List_Create(1, List_Nbr(c->Control_Points) / 10, sizeof(int));
+  Vertex *pv;
+  for (int i = 0; i < List_Nbr(c->Control_Points); i++){
+    List_Read(c->Control_Points, i, &pv);
+    List_Add(new_list, &pv->Num);
+    if(v_break.find(pv->Num) != v_break.end() && List_Nbr(new_list) > 1){
+      if(last_periodic)
+        break;
+      if(!(is_periodic&&first_periodic)){
+        Curve *cnew = _create_splitted_curve(c, new_list);
+        List_Add(shapes, &cnew);
+      }
+      first_periodic = false;
+      List_Reset(new_list);
+      List_Add(new_list, &pv->Num);
+    }
+    if( i== (List_Nbr(c->Control_Points) - 1) && is_periodic && ! first_periodic){
+      i = 0;
+      last_periodic = true;
+    }
+  }
+  if(List_Nbr(new_list) > 1){
+    Curve *cnew = _create_splitted_curve(c, new_list);
+    List_Add(shapes, &cnew);
+  }
+  // replace original curve by the new curves in all surfaces (and for
+  // the opposite curve)
+  List_T *rshapes = List_Create(2, 1, sizeof(Shape));
+  int N = List_Nbr(shapes);
+  for(int i = 0; i < List_Nbr(shapes); i++){
+    Curve *cc, *rcc;
+    List_Read(shapes, N - i - 1, &cc);
+    rcc=FindCurve(-cc->Num);
+    List_Add(rshapes, &rcc);
+  }
+  List_T *Surfs = Tree2List(GModel::current()->getGEOInternals()->Surfaces);
+  for(int i = 0; i < List_Nbr(Surfs); i++) {
+    Surface *s;
+    List_Read(Surfs, i, &s);
+    for(int j = 0; j < List_Nbr(s->Generatrices); j++) {
+      Curve *surface_curve;
+      List_Read(s->Generatrices, j, &surface_curve);
+      if(surface_curve->Num == c->Num){
+        List_Remove(s->Generatrices, j);
+        List_Insert_In_List(shapes, j, s->Generatrices);
+      }else if(surface_curve->Num == -c->Num){
+        List_Remove(s->Generatrices, j);
+        List_Insert_In_List(rshapes, j, s->Generatrices);
+      }
+    }
+  }
+  List_Delete(Surfs);
+  DeleteShape(c->Typ, c->Num);
+  List_Delete(new_list);
+  List_Delete(rshapes);
+  return true;
+}
+
 // Intersect a curve with a surface
 
 static void intersectCS(int N, double x[], double res[])
