@@ -2,32 +2,14 @@
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
-// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
-// 
-// Please report all bugs and problems to <gmsh@geuz.org>.
 
 #include "MFace.h"
 
 #if defined(HAVE_GMSH_EMBEDDED)
-#  include "GmshEmbedded.h"
+#include "GmshEmbedded.h"
 #else
-#  include "Numeric.h"
-#  include "Context.h"
+#include "Numeric.h"
+#include "Context.h"
 #endif
 
 extern Context_T CTX;
@@ -104,3 +86,23 @@ SVector3 MFace::normal() const
   return SVector3(n[0], n[1], n[2]);
 }
 
+bool MFace::computeCorrespondence(const MFace& other,int& rotation,bool& swap) const {
+  
+  rotation = 0;
+  swap = false;
+  
+  if (*this == other) {
+    for (int i=0;i<getNumVertices();i++) {
+      if (_v[0] == other.getVertex(i)) {
+        rotation = i;
+        break;
+      }
+    }
+    if (_v[1] == other.getVertex((rotation+1)%getNumVertices())) swap = false;
+    else                                                         swap = true;
+    return true;
+  }
+  return false;
+}
+
+  
