@@ -23,6 +23,9 @@ class MFace {
   inline int getNumVertices() const { return _v[3] ? 4 : 3; }
   inline MVertex *getVertex(const int i) const { return _v[i]; }
   inline MVertex *getSortedVertex(const int i) const { return _v[int(_si[i])]; }
+  
+  bool computeCorrespondence(const MFace&,int&,bool&) const;
+
   void getOrderedVertices(std::vector<MVertex*> &verts) const
   {
     for(int i = 0; i < getNumVertices(); i++)
@@ -91,13 +94,26 @@ class MFace {
   }
 };
 
+inline bool operator==(const MFace &f1, const MFace &f2)
+{
+  return (f1.getSortedVertex(0) == f2.getSortedVertex(0) &&
+          f1.getSortedVertex(1) == f2.getSortedVertex(1) &&
+          f1.getSortedVertex(2) == f2.getSortedVertex(2) &&
+          f1.getSortedVertex(3) == f2.getSortedVertex(3));
+}
+
+inline bool operator!=(const MFace &f1, const MFace &f2)
+{
+  return (f1.getSortedVertex(0) != f2.getSortedVertex(0) ||
+          f1.getSortedVertex(1) != f2.getSortedVertex(1) ||
+          f1.getSortedVertex(2) != f2.getSortedVertex(2) ||
+          f1.getSortedVertex(3) != f2.getSortedVertex(3));
+}
+
 struct Equal_Face : public std::binary_function<MFace, MFace, bool> {
   bool operator()(const MFace &f1, const MFace &f2) const
   {
-    return (f1.getSortedVertex(0) == f2.getSortedVertex(0) &&
-            f1.getSortedVertex(1) == f2.getSortedVertex(1) &&
-            f1.getSortedVertex(2) == f2.getSortedVertex(2) &&
-            f1.getSortedVertex(3) == f2.getSortedVertex(3));
+    return (f1 == f2);
   }
 };
 
@@ -115,4 +131,7 @@ struct Less_Face : public std::binary_function<MFace, MFace, bool> {
   }
 };
 
+
+
+
 #endif
