@@ -3,8 +3,9 @@
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
+#include <limits>
 #include "GModel.h"
-#include "Message.h"
+#include "GmshMessage.h"
 #include "OCCEdge.h"
 #include "OCCFace.h"
 #include "Context.h"
@@ -29,6 +30,23 @@ OCCEdge::OCCEdge(GModel *model, TopoDS_Edge edge, int num, GVertex *v1, GVertex
   // build the reverse curve
   c_rev = c;
   c_rev.Reverse();
+
+
+//   if (v0 == v1){
+//     const int JJ = 52;
+//     for (int i=1;i<JJ;i++){
+//       const double t = i/((double) JJ);
+//       const double xi = s0 + (s1-s0) * t; 
+//       GPoint p = point(xi);
+//       MEdgeVertex *v = new MEdgeVertex (p.x(),p.y(),p.z(),this,xi);
+//       mesh_vertices.push_back(v);
+//       meshAttributes.Method = MESH_NONE;
+//       if (i == 1)lines.push_back(new MLine(v1->mesh_vertices[0],v));
+//       else if (i == JJ-1)lines.push_back(new MLine(v,v2->mesh_vertices[0]));
+//       else lines.push_back(new MLine(mesh_vertices[i-1],v));
+//     }
+//   }
+
 }
 
 Range<double> OCCEdge::parBounds(int i) const
@@ -91,6 +109,7 @@ SPoint2 OCCEdge::reparamOnFace(GFace *face, double epar, int dir) const
 // True if the edge is a seam for the given face
 bool OCCEdge::isSeam(GFace *face) const
 {
+  if (face->geomType() == GEntity::CompoundSurface)return false; 
   const TopoDS_Face *s = (TopoDS_Face*) face->getNativePtr();
   BRepAdaptor_Surface surface(*s);
   //  printf("asking if edge %d is a seam of face %d\n",tag(),face->tag());
@@ -130,7 +149,7 @@ SVector3 OCCEdge::firstDer(double par) const
 double OCCEdge::parFromPoint(const SPoint3 &pt) const
 {
   Msg::Error("parFromPoint not implemented for OCCEdge");
-  return 0.;
+  return std::numeric_limits<double>::max();
 }
 
 GEntity::GeomType OCCEdge::geomType() const
