@@ -4,7 +4,7 @@
 // bugs and problems to <gmsh@geuz.org>.
 
 #include <math.h>
-#include "Message.h"
+#include "GmshMessage.h"
 #include "GmshUI.h"
 #include "GModel.h"
 #include "MElement.h"
@@ -99,7 +99,9 @@ static bool isElementVisible(MElement *ele)
   if(!ele->getVisibility()) return false;
   if(CTX.mesh.quality_sup) {
     double q;
-    if(CTX.mesh.quality_type == 2)
+    if(CTX.mesh.quality_type == 3)
+      q = ele->distoShapeMeasure();
+    else if(CTX.mesh.quality_type == 2)
       q = ele->rhoShapeMeasure();
     else if(CTX.mesh.quality_type == 1)
       q = ele->etaShapeMeasure();
@@ -363,6 +365,56 @@ static void drawBarycentricDual(std::vector<T*> &elements)
   gl2psDisable(GL2PS_LINE_STIPPLE);
 }
 
+template<class T>
+static void drawVoronoiDual(std::vector<T*> &elements)
+{
+  glColor4ubv((GLubyte *) & CTX.color.fg);
+  glEnable(GL_LINE_STIPPLE);
+  glLineStipple(1, 0x0F0F);
+  gl2psEnable(GL2PS_LINE_STIPPLE);
+  glBegin(GL_LINES);
+  for(unsigned int i = 0; i < elements.size(); i++){
+    T *ele = elements[i];
+    if(!isElementVisible(ele)) continue;
+    SPoint3 pc = ele->circumcenter();
+    if(ele->getDim() == 2){
+      for(int j = 0; j < ele->getNumEdges(); j++){
+        MEdge e = ele->getEdge(j);
+	SVector3 p2p1(e.getVertex(1)->x() - e.getVertex(0)->x(),
+		      e.getVertex(1)->y() - e.getVertex(0)->y(),
+		      e.getVertex(1)->z() - e.getVertex(0)->z());
+	SVector3 pcp1(pc.x() - e.getVertex(0)->x(),
+		      pc.y() - e.getVertex(0)->y(),
+		      pc.z() - e.getVertex(0)->z());
+	double alpha = dot(pcp1,p2p1) / dot(p2p1,p2p1);
+        SPoint3 p((1 - alpha)*e.getVertex(0)->x() + alpha * e.getVertex(1)->x(), 
+		  (1 - alpha)*e.getVertex(0)->y() + alpha * e.getVertex(1)->y(),
+		  (1 - alpha)*e.getVertex(0)->z() + alpha * e.getVertex(1)->z());
+	glVertex3d(pc.x(), pc.y(), pc.z());
+        glVertex3d(p.x(), p.y(), p.z());
+      }
+    }
+    else if(ele->getDim() == 3){
+      for(int j = 0; j < ele->getNumFaces(); j++){
+        MFace f = ele->getFace(j);
+        SPoint3 p = f.barycenter();
+        glVertex3d(pc.x(), pc.y(), pc.z());
+        glVertex3d(p.x(), p.y(), p.z());
+        for(int k = 0; k < f.getNumVertices(); k++){
+          MEdge e(f.getVertex(k), (k == f.getNumVertices() - 1) ? 
+                  f.getVertex(0) : f.getVertex(k + 1));
+          SPoint3 pe = e.barycenter();
+          glVertex3d(p.x(), p.y(), p.z());
+          glVertex3d(pe.x(), pe.y(), pe.z());
+        }
+      }
+    }
+  }
+  glEnd();
+  glDisable(GL_LINE_STIPPLE);
+  gl2psDisable(GL2PS_LINE_STIPPLE);
+}
+
 // Routine to fill the smooth normal container
 
 template<class T>
@@ -451,11 +503,13 @@ static void addElementsInArrays(GEntity *e, std::vector<T*> &elements,
 static void drawArrays(GEntity *e, VertexArray *va, GLint type, bool useNormalArray, 
                        int forceColor=0, unsigned int color=0)
 {
-  if(!va) return;
+  if(!va || !va->getNumVertices()) return;
 
   // If we want to be enable picking of individual elements we need to
   // draw each one separately
-  if(CTX.render_mode == GMSH_SELECT && CTX.pick_elements) {
+  bool select = (CTX.render_mode == GMSH_SELECT && CTX.pick_elements && 
+                 e->model() == GModel::current());
+  if(select) {
     if(va->getNumElementPointers() == va->getNumVertices()){
       for(int i = 0; i < va->getNumVertices(); i += va->getNumVerticesPerElement()){
         glPushName(va->getNumVerticesPerElement());
@@ -472,13 +526,11 @@ static void drawArrays(GEntity *e, VertexArray *va, GLint type, bool useNormalAr
   }
 
   glVertexPointer(3, GL_FLOAT, 0, va->getVertexArray());
-  glNormalPointer(GL_BYTE, 0, va->getNormalArray());
-  glColorPointer(4, GL_UNSIGNED_BYTE, 0, va->getColorArray());
-  
   glEnableClientState(GL_VERTEX_ARRAY);
 
   if(useNormalArray){
     glEnable(GL_LIGHTING);
+    glNormalPointer(GL_BYTE, 0, va->getNormalArray());
     glEnableClientState(GL_NORMAL_ARRAY);
   }
   else
@@ -491,6 +543,7 @@ static void drawArrays(GEntity *e, VertexArray *va, GLint type, bool useNormalAr
   else if(CTX.pick_elements || 
           (!e->getSelection() && (CTX.mesh.color_carousel == 0 || 
                                   CTX.mesh.color_carousel == 3))){
+    glColorPointer(4, GL_UNSIGNED_BYTE, 0, va->getColorArray());
     glEnableClientState(GL_COLOR_ARRAY);
   }
   else{
@@ -520,7 +573,8 @@ class drawMeshGVertex {
   {  
     if(!v->getVisibility()) return;
     
-    if(CTX.render_mode == GMSH_SELECT) {
+    bool select = (CTX.render_mode == GMSH_SELECT && v->model() == GModel::current());
+    if(select) {
       glPushName(0);
       glPushName(v->tag());
     }
@@ -528,7 +582,7 @@ class drawMeshGVertex {
     if(CTX.mesh.points || CTX.mesh.points_num)
       drawVerticesPerEntity(v);
 
-    if(CTX.render_mode == GMSH_SELECT) {
+    if(select) {
       glPopName();
       glPopName();
     }
@@ -569,8 +623,9 @@ class drawMeshGEdge {
   void operator () (GEdge *e)
   {  
     if(!e->getVisibility()) return;
-    
-    if(CTX.render_mode == GMSH_SELECT) {
+
+    bool select = (CTX.render_mode == GMSH_SELECT && e->model() == GModel::current());    
+    if(select) {
       glPushName(1);
       glPushName(e->tag());
     }
@@ -591,7 +646,7 @@ class drawMeshGEdge {
     if(CTX.mesh.tangents)
       drawTangents(e->lines);
 
-    if(CTX.render_mode == GMSH_SELECT) {
+    if(select) {
       glPopName();
       glPopName();
     }
@@ -662,7 +717,8 @@ class drawMeshGFace {
   {  
     if(!f->getVisibility()) return;
 
-    if(CTX.render_mode == GMSH_SELECT) {
+    bool select = (CTX.render_mode == GMSH_SELECT && f->model() == GModel::current());
+    if(select) {
       glPushName(2);
       glPushName(f->tag());
     }
@@ -696,8 +752,11 @@ class drawMeshGFace {
       if(CTX.mesh.triangles) drawBarycentricDual(f->triangles);
       if(CTX.mesh.quadrangles) drawBarycentricDual(f->quadrangles);
     }
+    else if(CTX.mesh.voronoi) {
+      if(CTX.mesh.triangles) drawVoronoiDual(f->triangles);
+    }
 
-    if(CTX.render_mode == GMSH_SELECT) {
+    if(select) {
       glPopName();
       glPopName();
     }
@@ -781,7 +840,8 @@ class drawMeshGRegion {
   {  
     if(!r->getVisibility()) return;
 
-    if(CTX.render_mode == GMSH_SELECT) {
+    bool select = (CTX.render_mode == GMSH_SELECT && r->model() == GModel::current());
+    if(select) {
       glPushName(3);
       glPushName(r->tag());
     }
@@ -823,7 +883,7 @@ class drawMeshGRegion {
       if(CTX.mesh.pyramids) drawBarycentricDual(r->pyramids);
     }
 
-    if(CTX.render_mode == GMSH_SELECT) {
+    if(select) {
       glPopName();
       glPopName();
     }
@@ -835,17 +895,18 @@ class drawMeshGRegion {
 
 void Draw_Mesh()
 {
-  GModel *m = GModel::current();
+  if(!CTX.mesh.draw) return;
 
   // make sure to flag any model-dependent post-processing view as
   // changed if the underlying mesh has, before resetting the changed
   // flag
-  for(unsigned int i = 0; i < PView::list.size(); i++)
-    if(PView::list[i]->getData()->hasModel(m) && CTX.mesh.changed)
-      PView::list[i]->setChanged(true);
-  
-  if(!CTX.mesh.draw) return;
-  
+  for(unsigned int i = 0; i < GModel::list.size(); i++){
+    GModel *m = GModel::list[i];
+    for(unsigned int j = 0; j < PView::list.size(); j++)
+      if(PView::list[j]->getData()->hasModel(m) && CTX.mesh.changed)
+        PView::list[j]->setChanged(true);
+  }
+
   glPointSize(CTX.mesh.point_size);
   gl2psPointSize(CTX.mesh.point_size * CTX.print.eps_point_size_factor);
 
@@ -864,33 +925,40 @@ void Draw_Mesh()
       else
 	glDisable((GLenum)(GL_CLIP_PLANE0 + i));
   }
-  
+
   static bool busy = false;
   if(!busy){
     busy = true;
-    int status = m->getMeshStatus();
-    if(CTX.mesh.changed) {
-      Msg::Debug("Mesh has changed: reinitializing drawing data", CTX.mesh.changed);
-      if(status >= 1 && CTX.mesh.changed & ENT_LINE)
-        std::for_each(m->firstEdge(), m->lastEdge(), initMeshGEdge());
-      if(status >= 2 && CTX.mesh.changed & ENT_SURFACE){
-        if(m->normals) delete m->normals;
-        m->normals = new smooth_normals(CTX.mesh.angle_smooth_normals);
-        if(CTX.mesh.smooth_normals)
-          std::for_each(m->firstFace(), m->lastFace(), initSmoothNormalsGFace());
-        std::for_each(m->firstFace(), m->lastFace(), initMeshGFace());
+
+    for(unsigned int i = 0; i < GModel::list.size(); i++){
+      GModel *m = GModel::list[i];
+      if(CTX.draw_all_models || m == GModel::current()){
+        int status = m->getMeshStatus();
+        if(CTX.mesh.changed) {
+          Msg::Debug("Mesh has changed: reinitializing drawing data", CTX.mesh.changed);
+          if(status >= 1 && CTX.mesh.changed & ENT_LINE)
+            std::for_each(m->firstEdge(), m->lastEdge(), initMeshGEdge());
+          if(status >= 2 && CTX.mesh.changed & ENT_SURFACE){
+            if(m->normals) delete m->normals;
+            m->normals = new smooth_normals(CTX.mesh.angle_smooth_normals);
+            if(CTX.mesh.smooth_normals)
+              std::for_each(m->firstFace(), m->lastFace(), initSmoothNormalsGFace());
+            std::for_each(m->firstFace(), m->lastFace(), initMeshGFace());
+          }
+          if(status >= 3 && CTX.mesh.changed & ENT_VOLUME)
+            std::for_each(m->firstRegion(), m->lastRegion(), initMeshGRegion());
+        }
+        if(status >= 0)
+          std::for_each(m->firstVertex(), m->lastVertex(), drawMeshGVertex());
+        if(status >= 1)
+          std::for_each(m->firstEdge(), m->lastEdge(), drawMeshGEdge());
+        if(status >= 2)
+          std::for_each(m->firstFace(), m->lastFace(), drawMeshGFace());
+        if(status >= 3)
+          std::for_each(m->firstRegion(), m->lastRegion(), drawMeshGRegion());
       }
-      if(status >= 3 && CTX.mesh.changed & ENT_VOLUME)
-        std::for_each(m->firstRegion(), m->lastRegion(), initMeshGRegion());
     }
-    if(status >= 0)
-      std::for_each(m->firstVertex(), m->lastVertex(), drawMeshGVertex());
-    if(status >= 1)
-      std::for_each(m->firstEdge(), m->lastEdge(), drawMeshGEdge());
-    if(status >= 2)
-      std::for_each(m->firstFace(), m->lastFace(), drawMeshGFace());
-    if(status >= 3)
-      std::for_each(m->firstRegion(), m->lastRegion(), drawMeshGRegion());
+
     CTX.mesh.changed = 0;
     busy = false;
   }
