@@ -1,23 +1,7 @@
-// $Id: gl2gif.cpp,v 1.26 2008-06-07 17:20:47 geuzaine Exp $
+// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
-// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
-// 
-// Please report all bugs and problems to <gmsh@geuz.org>.
+// See the LICENSE.txt file for license information. Please report all
+// bugs and problems to <gmsh@geuz.org>.
 
 /* 
  * Warning: This code is really a dirty hack. It SHOULD be cleaned
@@ -77,7 +61,7 @@ static int static_nbcolors;
 static pixel **static_pixels;
 static colorhash_table static_cht;
 
-colorhash_table ppm_alloccolorhash()
+static colorhash_table ppm_alloccolorhash()
 {
   colorhash_table cht;
   int i;
@@ -90,7 +74,7 @@ colorhash_table ppm_alloccolorhash()
   return cht;
 }
 
-void ppm_freecolorhash(colorhash_table cht)
+static void ppm_freecolorhash(colorhash_table cht)
 {
   int i;
   colorhist_list chl, chlnext;
@@ -103,9 +87,9 @@ void ppm_freecolorhash(colorhash_table cht)
   Free((char *)cht);
 }
 
-colorhash_table ppm_computecolorhash(pixel ** const pixels,
-                                     const int cols, const int rows,
-                                     const int maxcolors, int *const colorsP)
+static colorhash_table ppm_computecolorhash(pixel ** const pixels,
+					    const int cols, const int rows,
+					    const int maxcolors, int *const colorsP)
 {
   colorhash_table cht;
   const pixel *pP;
@@ -140,8 +124,8 @@ colorhash_table ppm_computecolorhash(pixel ** const pixels,
   return cht;
 }
 
-int ppm_addtocolorhash(colorhash_table cht, const pixel * const colorP,
-                       const int value)
+static int ppm_addtocolorhash(colorhash_table cht, const pixel * const colorP,
+			      const int value)
 {
   register int hash;
   register colorhist_list chl;
@@ -155,8 +139,8 @@ int ppm_addtocolorhash(colorhash_table cht, const pixel * const colorP,
   return 0;
 }
 
-colorhist_vector ppm_colorhashtocolorhist(const colorhash_table cht,
-                                          const int maxcolors)
+static colorhist_vector ppm_colorhashtocolorhist(const colorhash_table cht,
+						 const int maxcolors)
 {
   colorhist_vector chv;
   colorhist_list chl;
@@ -177,8 +161,8 @@ colorhist_vector ppm_colorhashtocolorhist(const colorhash_table cht,
   return chv;
 }
 
-colorhash_table ppm_colorhisttocolorhash(const colorhist_vector chv,
-                                         const int colors)
+static colorhash_table ppm_colorhisttocolorhash(const colorhist_vector chv,
+						const int colors)
 {
   colorhash_table cht;
   int i, hash;
@@ -204,9 +188,9 @@ colorhash_table ppm_colorhisttocolorhash(const colorhist_vector chv,
   return cht;
 }
 
-colorhist_vector ppm_computecolorhist(pixel ** const pixels,
-                                      const int cols, const int rows,
-                                      const int maxcolors, int *const colorsP)
+static colorhist_vector ppm_computecolorhist(pixel ** const pixels,
+					     const int cols, const int rows,
+					     const int maxcolors, int *const colorsP)
 {
   colorhash_table cht;
   colorhist_vector chv;
@@ -220,7 +204,7 @@ colorhist_vector ppm_computecolorhist(pixel ** const pixels,
 }
 
 
-int ppm_lookupcolor(const colorhash_table cht, const pixel * const colorP)
+static int ppm_lookupcolor(const colorhash_table cht, const pixel * const colorP)
 {
   int hash;
   colorhist_list chl;
@@ -233,7 +217,7 @@ int ppm_lookupcolor(const colorhash_table cht, const pixel * const colorP)
   return -1;
 }
 
-void ppm_freecolorhist(colorhist_vector chv)
+static void ppm_freecolorhist(colorhist_vector chv)
 {
   Free((char *)chv);
 }
