@@ -6,7 +6,7 @@
 #include "meshGRegionLocalMeshMod.h"
 #include "GEntity.h"
 #include "GRegion.h"
-#include "Message.h"
+#include "GmshMessage.h"
 #include "Numeric.h"
 
 static int edges[6][2] =    {{0,1},{0,2},{0,3},{1,2},{1,3},{2,3}};
