@@ -11,9 +11,14 @@ class BDS_Face;
 class MVertex;
 class MTriangle;
 class MTetrahedron;
+class MElement;
+
 enum gmshQualityMeasure4Triangle {QMTRI_RHO, QMTRI_COND};
 enum gmshQualityMeasure4Tet{QMTET_1, QMTET_2, QMTET_3, QMTET_ONE, QMTET_COND};
 
+double qmDistorsionOfMapping (MTriangle *e);
+double qmDistorsionOfMapping (MTetrahedron *e);
+
 double qmTriangle(MTriangle *f, const gmshQualityMeasure4Triangle &cr); 
 double qmTriangle(BDS_Face *f, const gmshQualityMeasure4Triangle &cr); 
 double qmTriangle(const BDS_Point *p1, const BDS_Point *p2, const BDS_Point *p3, 
