@@ -37,8 +37,8 @@ void GMSH_EvaluatePlugin::getName(char *name) const
 void GMSH_EvaluatePlugin::getInfos(char *author, char *copyright,
                                    char *help_text) const
 {
-  strcpy(author, "C. Geuzaine");
-  strcpy(copyright, "DGR (www.multiphysics.com)");
+  strcpy(author, "C. Geuzaine, J.-F. Remacle");
+  strcpy(copyright, "C. Geuzaine, J.-F. Remacle");
   strcpy(help_text,
          "Plugin(Evaluate) sets the `Component'-th\n"
          "component of the `TimeStep'-th time step in the\n"
