@@ -30,8 +30,8 @@ void GMSH_ExtractElementsPlugin::getName(char *name) const
 
 void GMSH_ExtractElementsPlugin::getInfos(char *author, char *copyright, char *help_text) const
 {
-  strcpy(author, "C. Geuzaine");
-  strcpy(copyright, "DGR (www.multiphysics.com)");
+  strcpy(author, "C. Geuzaine, J.-F. Remacle");
+  strcpy(copyright, "C. Geuzaine, J.-F. Remacle");
   strcpy(help_text,
          "Plugin(ExtractElements) extracts the elements\n"
          "from the view `iView' whose `TimeStep'-th values\n"
