@@ -30,8 +30,8 @@ void GMSH_MakeSimplexPlugin::getName(char *name) const
 void GMSH_MakeSimplexPlugin::getInfos(char *author, char *copyright,
                                       char *help_text) const
 {
-  strcpy(author, "C. Geuzaine");
-  strcpy(copyright, "DGR (www.multiphysics.com)");
+  strcpy(author, "C. Geuzaine, J.-F. Remacle");
+  strcpy(copyright, "C. Geuzaine, J.-F. Remacle");
   strcpy(help_text,
          "Plugin(MakeSimplex) decomposes all non-\n"
          "simplectic elements (quadrangles, prisms,\n"
