@@ -38,8 +38,8 @@ void GMSH_RemovePlugin::getName(char *name) const
 void GMSH_RemovePlugin::getInfos(char *author, char *copyright,
                                    char *help_text) const
 {
-  strcpy(author, "C. Geuzaine");
-  strcpy(copyright, "DGR (www.multiphysics.com)");
+  strcpy(author, "C. Geuzaine, J.-F. Remacle");
+  strcpy(copyright, "C. Geuzaine, J.-F. Remacle");
   strcpy(help_text,
          "Plugin(Remove) removes the marked items\n"
          "from the view `iView'. If `iView' < 0, the plugin\n"
