@@ -1,25 +1,11 @@
+// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+//
+// See the LICENSE.txt file for license information. Please report all
+// bugs and problems to <gmsh@geuz.org>.
+
 #ifndef _OCTREE_POST_H_
 #define _OCTREE_POST_H_
 
-// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
-// 
-// Please report all bugs and problems to <gmsh@geuz.org>.
-
 #include "Octree.h"
 
 class PView;
@@ -36,7 +22,6 @@ class OctreePost
   Octree *_SH, *_VH, *_TH;
   Octree *_SI, *_VI, *_TI;
   Octree *_SY, *_VY, *_TY;
-  Octree *_GModel;
   PView *_theView;
   PViewDataList *_theViewDataList;
   PViewDataGModel *_theViewDataGModel;
