@@ -2,15 +2,12 @@
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
-//
-// Contributor(s):
-// 
 
 #include "PViewData.h"
 #include "ListUtils.h"
 #include "adaptiveData.h"
 #include "Numeric.h"
-#include "Message.h"
+#include "GmshMessage.h"
 
 PViewData::PViewData()
   : _dirty(true), _fileIndex(0), _adaptive(0)
@@ -28,8 +25,8 @@ bool PViewData::finalize()
     Msg::Info("Initializing adaptive data %p interp size=%d",
 	this, _interpolation.size());
     _adaptive = new adaptiveData(this);
-    _adaptive->initWithLowResolution(0);
   }
+  if(_adaptive) _adaptive->initWithLowResolution(0);
   _dirty = false;
   return true;
 }
