@@ -2,14 +2,11 @@
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
-//
-// Contributor(s):
-// 
 
 #include "PViewDataGModel.h"
 #include "MElement.h"
 #include "Numeric.h"
-#include "Message.h"
+#include "GmshMessage.h"
 
 PViewDataGModel::PViewDataGModel(DataType type) 
   : PViewData(), _min(VAL_INF), _max(-VAL_INF), _type(type)
@@ -128,6 +125,16 @@ int PViewDataGModel::getNumTensors(int step)
   return 0;
 }
 
+int PViewDataGModel::getNumPoints(int step)
+{
+  if(_steps.empty()) return 0;
+  GModel *m = _steps[0]->getModel(); // to generalize
+  int n = 0;
+  for(GModel::viter it = m->firstVertex(); it != m->lastVertex(); ++it)
+    n += (*it)->points.size();
+  return n;
+}
+
 int PViewDataGModel::getNumLines(int step)
 {
   if(_steps.empty()) return 0;
@@ -252,9 +259,9 @@ int PViewDataGModel::getNode(int step, int ent, int ele, int nod,
 	vy[i] = e->getVertex(i)->y();
 	vz[i] = e->getVertex(i)->z();
       }
-      x = e->interpolate(vx, p[3 * nod], p[3 * nod + 1], p[3 * nod + 2]);
-      y = e->interpolate(vy, p[3 * nod], p[3 * nod + 1], p[3 * nod + 2]);
-      z = e->interpolate(vz, p[3 * nod], p[3 * nod + 1], p[3 * nod + 2]);
+      x = e->interpolate(vx, p[3 * nod], p[3 * nod + 1], p[3 * nod + 2], 1, 1);
+      y = e->interpolate(vy, p[3 * nod], p[3 * nod + 1], p[3 * nod + 2], 1, 1);
+      z = e->interpolate(vz, p[3 * nod], p[3 * nod + 1], p[3 * nod + 2], 1, 1);
     }
     return 0;
   }
