@@ -2,12 +2,9 @@
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
-//
-// Contributor(s):
-// 
 
 #include <string.h>
-#include "Message.h"
+#include "GmshMessage.h"
 #include "Numeric.h"
 #include "PViewData.h"
 
